
library(startR)
library(multiApply)
library(CSTools)
library(s2dv)
source('/esarchive/scratch/cdelgado/gitlab/cdelgado_copernicus/C3S_34c/metrics/load_and_average_monthly.R')
source('/esarchive/scratch/cdelgado/gitlab/cdelgado_copernicus/C3S_34c/metrics/load_and_average_daily.R')
source('/esarchive/scratch/cdelgado/gitlab/cdelgado_copernicus/C3S_34c/metrics/change_clim_ref_period.R')
source('/esarchive/scratch/cdelgado/gitlab/cdelgado_copernicus/C3S_34c/metrics/get_days_dcpp.R')

################################################################################################
########################################## PARAMETERS ##########################################
################################################################################################

retrieve <- TRUE
max_jobs <- 60
ncores <- 8
num_procs_start_call <- 1
save_outputs = TRUE
outputs_path = '/esarchive/scratch/cdelgado/copernicus_outputs/outputs_load_avg/'
if (save_outputs){if (!dir.exists(outputs_path)){dir.create(file.path(outputs_path), recursive = TRUE)}}

## MODEL, REANALYSIS AND VARIABLE
models_dcpp <- c('EC-Earth3-i1','HadGEM3','MPI-ESM1.2-HR','CMCC-CM2-SR5')
models_WMO_dcpp <- c('BCC-CSM2-MR','CanESM5','CESM1-1-CAM5-CMIP5','IPSL-CM6A-LR','MIROC6','NorCPM1','MRI-ESM2-0','EC-Earth3-i2','MPI-ESM1.2-LR')
models_hist <- paste0(models_dcpp,'_hist')
models_WMO_hist <- paste0(models_WMO_dcpp,'_hist')
reanalyses <- c('JRA-55','NCEP1','ERA5')
observations <- c('GHCNv4','GISTEMPv4','HadCRUT4','GPCC','HadSLP2')
variables <- c('tas','psl','pr','tos','tas-tos','tasmin','tasmax')
seasons <- c('Annual','DJF','JJA')
different_fyears <- list(1, 1:5, 6:10, 1:10)

dataset <- models_WMO_hist[3] # the actual dataset that is being processed and saved

for (fyears in different_fyears[[1]]){ ## only useful for models_dcpp and models_WMO_dcpp
  for (variable in variables[c(1,3)]){
    for (season in seasons[1]){
      
      if (dataset %in% c(models_dcpp,models_WMO_dcpp) & length(fyears)==1){
        print(paste0('Starting: ', dataset, ' - ', variable, ' - ', season, ' - fy', fyears))
      } else if (dataset %in% c(models_dcpp,models_WMO_dcpp) & length(fyears)>1){
        print(paste0('Starting: ', dataset, ' - ', variable, ' - ', season, ' - fy', fyears[1],'-',fyears[length(fyears)]))
      } else {
        print(paste0('Starting ', dataset, ' - ', variable, ' - ', season))
      }
      
      ## REGION
      lat_min <- -90
      lat_max <- 90
      lon_min <- 0
      lon_max <- 359.9
      
      members <- 'all'
      
      ## PERIOD
      if (dataset %in% c('HadSLP2','ERA5')){
        years_obs <- 1960:2018
      } else {
        years_obs <- 1960:2019
      }
      
      years_hist <- 1961:2014
      
      if(dataset %in% c('CMCC-CM2-SR5','CESM1-1-CAM5-CMIP5')){
        sdates <- 1960:2015
      } else if (dataset == 'BCC-CSM2-MR'){
        sdates <- 1961:2014
      } else if (dataset %in% c('CanESM5','IPSL-CM6A-LR')){
        sdates <- 1960:2016
      } else if (dataset %in% c('EC-Earth3-i2','MRI-ESM2-0')){
        sdates <- 1960:2017
      } else {
        sdates <- 1960:2018
      }
      
      # if(dataset %in% c(models_dcpp,models_hist,models_WMO_dcpp,models_WMO_hist)){
      #   retrieve = FALSE
      # } else if (dataset %in% c(reanalyses, observations)){
      #   retrieve = TRUE
      # }
      
      if (dataset %in% c(models_WMO_dcpp,models_WMO_hist) & season != 'Annual'){
        stop('Only Annual means for WMO models since some of them are initialised in January')
      }
      
      if (variable %in% c('tas','tos','psl','pr','tas-tos')){
        
        freq <- 'monthly'
        
        ################################################################################################
        ############################## LOAD THE MONTHLY DATA ###########################################
        ################################################################################################
        
        if (dataset == 'EC-Earth3-i1'){
          
          # added another fyear for the last 2 months (init in N)
          fyears_ECEarth = fyears[1] : (fyears[length(fyears)]+1)
          
          if (variable %in% c('tas','pr','psl','tos')){
            path_exp = '/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/Amon/$var$/gr/v20190713/$var$_Amon_*_s$sdate$-$member$_gr_$fyear$.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data = Start(dataset = path_exp,
                       var = variable,
                       sdate = paste0(sdates),
                       fmonth = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       fyear = indices(fyears_ECEarth),
                       member = members,
                       fyear_depends = 'sdate',
                       fmonth_across = 'fyear',
                       merge_across_dims = TRUE,
                       synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (retrieve){
            lon = as.vector(attr(data,'Variables')$common[[variable]]$dim[[1]]$vals)
            lat = as.vector(attr(data,'Variables')$common[[variable]]$dim[[2]]$vals)
          } else {
            lon = as.vector(attr(data,'Variables')$dat1[['lon']])
            lat = as.vector(attr(data,'Variables')$dat1[['lat']])
          }
          
        } else if (dataset == 'HadGEM3'){
          
          # init in N
          fmonths = (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          
          if (variable %in% c('tas','pr','psl','ts')){
            path_exp = '/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/$member$/Amon/$var$/gr/v20200316/$var$_Amon_HadGEM3_dcppA-hindcast_s$sdate$-$member$_gr_$sdate$11-$aux$03.nc'
          } else if (variable == 'tos'){
            ## These files have been created with the following commands:
            ## for i in {1..10}; do python3 ../utils_python/interpolate.py /esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/r${i}i1p1f2/Omon/tos/gn/v20200417/ /esarchive/scratch/cdelgado/data/HadGEM3_tos_gr/ r432x324 bil: done
            ## mkdir HadGEM3_tos_gr_merged; for m in {1..10}; do for s in {1960..2018}; do cdo mergetime HadGEM3_tos_gr/tos_Omon_HadGEM3-GC31-MM_dcppA-hindcast_s${s}-r${m}i1p1f2_gn_* HadGEM3_tos_gr_merged/tos_Omon_HadGEM3-GC31-MM_dcppA-hindcast_s${s}-r${m}i1p1f2_gn_${s}11-$(($s+11))03.nc; done; done
            ## rm -r HadGEM3_tos_gr; mv HadGEM3_tos_gr_merged HadGEM3_tos_gr 
            stopifnot(identical(season,'Annual'))
            path_exp <- '/esarchive/scratch/cdelgado/data/HadGEM3_tos_gr/$var$_Omon_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-$member$_gn_$sdate$11-$aux$03.nc'
          } else {
            stop('Incorrect variable')
          }

          data = Start(dataset = path_exp,
                       var = variable,
                       sdate = paste0(sdates),
                       aux = 'all',
                       aux_depends = 'sdate',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       fmonth = indices(fmonths),
                       member = members,
                       synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call, retrieve = retrieve)
          
          if (variable == 'ts'){variable = 'tos'}
          
          if (retrieve){
            lon = as.vector(attr(data,'Variables')$common[[variable]]$dim[[1]]$vals)
            lat = as.vector(attr(data,'Variables')$common[[variable]]$dim[[2]]$vals)
          } else {
            lon = as.vector(attr(data,'Variables')$dat1[['lon']])
            lat = as.vector(attr(data,'Variables')$dat1[['lat']])
          }
          
        } else if (dataset == 'MPI-ESM1.2-HR'){
          
          # init in N
          fmonths = (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/dcppA-hindcast/$member$/Amon/$var$/gn/$version$/$var$_*_s$sdate$-$member$_gn_$sdate$11-$aux$12.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/dcppA-hindcast/$member$/Omon/$var$/gr/$version$/$var$_*_s$sdate$-$member$_gr_$sdate$11-$aux$12.nc'
          }
          data = Start(dataset = path_exp,
                       var = variable,
                       sdate = paste0(sdates),
                       version = 'all',
                       version_depends = 'member',
                       aux = 'all',
                       aux_depends = 'sdate',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       fmonth = indices(fmonths),
                       member = members,
                       synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (retrieve){
            lon = as.vector(attr(data,'Variables')$common[[variable]]$dim[[1]]$vals)
            lat = as.vector(attr(data,'Variables')$common[[variable]]$dim[[2]]$vals)
          } else {
            lon = as.vector(attr(data,'Variables')$dat1[['lon']])
            lat = as.vector(attr(data,'Variables')$dat1[['lat']])
          }
          
        } else if (dataset == 'CMCC-CM2-SR5'){
          
          # init in N
          fmonths = (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          if (variable == 'tos'){lat_max <- 91} ## if lat_max==90, it doesn't find the last latitude for this dataset
          
          if (variable %in% c('tas','psl','pr')){
            
            if (identical(members,'all')){
              members <- paste0('r',1:10,'i1p1f1')
              versions <- list('r1i1p1f1' = 'v20210312', 'r2i1p1f1' = 'v20210312', 'r3i1p1f1' = 'v20210312', 'r4i1p1f1' = 'v20210312', 'r5i1p1f1' = 'v20210312', 'r6i1p1f1' = 'v20210312', 
                               'r7i1p1f1' = 'v20210719', 'r8i1p1f1' = 'v20210719', 'r9i1p1f1' = 'v20210719', 'r10i1p1f1' = 'v20210719')
            }
            
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/cmcc-cm2-sr5/cmip6-dcppA-hindcast_i1p1/DCPP/CMCC/CMCC-CM2-SR5/dcppA-hindcast/$member$/Amon/$var$/gn/$version$/$var$_Amon_CMCC-CM2-SR5_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
            data <- Start(dataset = path_exp,
                          var = variable,
                          sdate = paste0(sdates),
                          aux = 'all',
                          aux_depends = 'sdate',
                          lat = values(list(lat_min, lat_max)),
                          lon = values(list(lon_min, lon_max)),
                          fmonth = indices(fmonths),
                          member = members,
                          version = versions,
                          version_depends = 'member',
                          synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                          return_vars = list(lat = 'dataset', lon = 'dataset'),
                          lat_reorder = Sort(decreasing = F),
                          lon_reorder = CircularSort(0, 360),
                          num_procs = num_procs_start_call , retrieve = retrieve)
            
          } else if (variable == 'tos'){
            
            path_exp <- '/esarchive/scratch/cdelgado/data/CMCC_tos_gr/$var$_Omon_CMCC-CM2-SR5_dcppA-hindcast_s$sdate$-$member$_gn_$sdate$11-$aux$12.nc'
            data <- Start(dataset = path_exp,
                          var = variable,
                          sdate = paste0(sdates),
                          aux = 'all',
                          aux_depends = 'sdate',
                          lat = values(list(lat_min, lat_max)),
                          lon = values(list(lon_min, lon_max)),
                          fmonth = indices(fmonths),
                          member = members,
                          synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                          return_vars = list(lat = 'dataset', lon = 'dataset'),
                          lat_reorder = Sort(decreasing = F),
                          lon_reorder = CircularSort(0, 360),
                          num_procs = num_procs_start_call , retrieve = retrieve)
            
          }
          
          lon = as.vector(attr(data,'Variables')$dat1[['lon']])
          lat = as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'BCC-CSM2-MR'){
          
          ## Init in January
          fmonths = (1 + 12*(fyears[1]-1)) : (12*fyears[length(fyears)])
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/bcc-csm2-mr/cmip6-dcppA-hindcast_i1p1/DCPP/BCC/BCC-CSM2-MR/dcppA-hindcast/$member$/Amon/$var$/gn/v20200101/$var$_Amon_BCC-CSM2-MR_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/bcc-csm2-mr/cmip6-dcppA-hindcast_i1p1/DCPP/BCC/BCC-CSM2-MR/dcppA-hindcast/$member$/Omon/$var$/gn/v20200101/$var$_Omon_BCC-CSM2-MR_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(fmonth = c('fmonth','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'CanESM5'){
          
          ## Init in January
          fmonths = (1 + 12*(fyears[1]-1)) : (12*fyears[length(fyears)])
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/canesm5/cmip6-dcppA-hindcast_i1p2/DCPP/CCCma/CanESM5/dcppA-hindcast/$member$i1p2f1/Amon/$var$/gn/v20190429/$var$_Amon_CanESM5_dcppA-hindcast_s$sdate$-$member$i1p2f1_gn_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/canesm5/cmip6-dcppA-hindcast_i1p2/DCPP/CCCma/CanESM5/dcppA-hindcast/$member$i1p2f1/Omon/$var$/gr/v20190429/$var$_Omon_CanESM5_dcppA-hindcast_s$sdate$-$member$i1p2f1_gr_$aux$.nc'
          }
          
          if (members == 'all'){
            members = paste0('r',1:20) # there are another 10 members, but they don't have these variables
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'CESM1-1-CAM5-CMIP5'){  ### NCAR DPLE
          
          # init in N
          fmonths = (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          
          if (variable == 'pr'){variable <- 'prlr'}
          
          if (variable %in% c('tas','psl','prlr')){
            path_exp <- '/esarchive/exp/ncar/cesm-dple/monthly_mean/$var$/$var$_$sdate$1101.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/ncar/cesm-dple/monthly_mean/$var$-lonlat/$var$_$sdate$1101.nc'
          } else {stop('Incorrect variable')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(member = c('member','ensemble'), fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'prlr'){variable <- 'pr'}
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'IPSL-CM6A-LR'){
          
          warning('Running locally. Compute() returns strange things')
          retrieve <- TRUE
          
          ## Init in January
          fmonths = (1 + 12*(fyears[1]-1)) : (12*fyears[length(fyears)])
          
          if (lat_min == -90 & lat_max == 90){lats = 'all'}
          if (lon_min == 0 & lon_max == 359.9){lons = 'all'}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/ipsl-cm6a-lr/cmip6-dcppA-hindcast_i1p1/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/Amon/$var$/gr/v20200504/$var$_Amon_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/ipsl-cm6a-lr/cmip6-dcppA-hindcast_i1p1/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/Omon/$var$/gr/v20200504/$var$_Omon_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
          } else {stop('Incorrect variable')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = lats, #values(list(lat_min, lat_max)),
                        lon = lons, #values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'MIROC6'){
          
          # init in N
          fmonths = (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          
          if (variable %in% c('tas')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/miroc6/cmip6-dcppA-hindcast_i1p1/DCPP/MIROC/MIROC6/dcppA-hindcast/$member$/Amon/$var$/gn/v20200417/$var$_Amon_MIROC6_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else if (variable %in% c('psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/miroc6/cmip6-dcppA-hindcast_i1p1/DCPP/MIROC/MIROC6/dcppA-hindcast/$member$/Amon/$var$/gn/v20200504/$var$_Amon_MIROC6_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/miroc6/cmip6-dcppA-hindcast_i1p1/DCPP/MIROC/MIROC6/dcppA-hindcast/$member$/Omon/$var$/gr/v20200416/$var$_Omon_MIROC6_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
          } else {stop('Incorrect variable')}
          
          if(variable == 'tas'){
            stop('here') # the results for the r9 was all NAs. Run it independently for each member (and not got NAs)
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'NorCPM1'){
          
          # init in Oct
          fmonths = (1 + 12*(fyears[1]-1)) : (3 + 12*fyears[length(fyears)])
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/norcpm1/cmip6-dcppA-hindcast_i1p1/DCPP/NCC/NorCPM1/dcppA-hindcast/$member$/Amon/$var$/gn/v20200320/$var$_Amon_NorCPM1_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/norcpm1/cmip6-dcppA-hindcast_i1p1/DCPP/NCC/NorCPM1/dcppA-hindcast/$member$/Omon/$var$/gr/v20200320/$var$_Omon_NorCPM1_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
          } else {stop('Incorrect variable')}
          
          if (lat_min == -90 & lat_max == 90){lats = 'all'}
          if (lon_min == 0 & lon_max == 359.9){lons = 'all'}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = lats,#values(list(lat_min, lat_max)),
                        lon = lons,#values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'MRI-ESM2-0'){
          
          if (any(fyears > 5)){stop('MRI system only provides data for the first five forecast years')}

          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/mri-esm2-0/cmip6-dcppA-hindcast_i1p1/DCPP/MRI/MRI-ESM2-0/dcppA-hindcast/$member$/Amon/$var$/gn/v20200101/$var$_Amon_MRI-ESM2-0_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/scratch/cdelgado/data/MRI_tos_gr/$var$_Omon_MRI-ESM2-0_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else {stop('Incorrect variable')}
          
          # init in Nov
          fmonths = (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        member = members,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'EC-Earth3-i2'){
          
          if (identical(members,'all')){members <- paste0('r',6:10,'i2p1f1')}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/ec-earth3/cmip6-dcppA-hindcast_i2p1/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/Amon/$var$/gr/v20200101/$var$_Amon_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$fyear$01-$fyear$12.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/scratch/cdelgado/data/EC-Earth3-i2_tos_gr/$var$_Omon_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gn_$fyear$01-$fyear$12.nc'
          } else {stop('Incorrect variable')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fyear = indices(fyears),
                        fmonth = 'all',
                        fyear_depends = 'sdate',
                        member = members,
                        fmonth_across = 'fyear',
                        merge_across_dims = TRUE,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'MPI-ESM1.2-LR'){
          
          # init in Nov
          fmonths <- (1 + 12*(fyears[1]-1)) : (2 + 12*fyears[length(fyears)])
          
          if (identical(members,'all')){members <- paste0('r',1:16,'i1p1f1')}
          if (identical(variable,'tos')){variable <- 'ts'}
          
          if (variable %in% c('tas','psl','pr','ts')){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-lr/cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-LR/dcppA-hindcast/$member$/Amon/$var$/gn/v20200101/$var$_Amon_MPI-ESM1-2-LR_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'
          } else {stop('Incorrect variable')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        sdate = paste0(sdates),
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fmonth = indices(fmonths),
                        aux = 'all',
                        aux_depends = 'sdate',
                        member = members,
                        synonims = list(fmonth = c('fmonth','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (identical(variable,'ts')){variable <- 'tos'}
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'JRA-55'){
          
          if (variable %in% c('tas','psl')){
            path_obs = '/esarchive/recon/jma/jra55/monthly_mean/$var$_f6h/$var$_$year$$month$.nc'
          } else if (variable == 'tos'){
            path_obs = '/esarchive/recon/jma/jra55/monthly_mean/$var$/$var$_$year$$month$.nc'
          } else if (variable == 'pr'){
            path_obs = '/esarchive/recon/jma/jra55/monthly_mean/$var$_s0-3h/$var$_$year$$month$.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data = Start(dataset = path_obs,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'GPCC'){
          
          if (variable == 'pr'){
            variable = 'prlr'
            path_obs = '/esarchive/obs/noaa/gpcc-v2018/monthly_mean/$var$/$var$_$year$$month$.nc'
          } else {
            stop('There is only data for prlr for NOAA-GPCC reanalysis')
          }
          
          data = Start(dataset = path_obs,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'prlr'){variable = 'pr'}
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'GHCNv2'){
          
          if (variable == 'tas'){
            path_obs = '/esarchive/obs/noaa/ghcn_v2/monthly_mean/$var$/$var$_$year$$month$.nc'
            warning('Only data up to 201502')
          } else {
            stop('Only tas for GHCNv2')
          }
          
          data = Start(dataset = path_obs,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'GHCNv4'){
          
          if (variable == 'tas'){
            path_obs = '/esarchive/obs/noaa/ghcn_v4/monthly_mean/$var$/$var$_$year$$month$.nc'
            variable = 'tasanomaly'
            warning('GHCNv4 provides tasanomaly')
          } else {
            stop('Only tasanomaly for GHCNv4')
          }
          
          data = Start(dataset = path_obs,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'tasanomaly'){
            variable = 'tas'
          }
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'GISTEMPv4'){
          
          if (variable == 'tas-tos'){
            variable = 'tasanomaly'
            path_obs = '/esarchive/obs/noaa-nasa/ghcnersstgiss/monthly_mean/$var$/$var$_$year$$month$.nc'
          } else {
            stop('Only tas_anomaly (tas-tos) for GISTEMPv4')
          }
          
          data = Start(dataset = path_obs,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'tasanomaly'){
            variable = 'tas-tos'
          }
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'HadCRUT4'){
          
          if (variable == 'tas-tos'){
            variable = 'tasanomaly'
            path_obs = '/esarchive/obs/ukmo/hadcrut_v4.6/monthly_mean/$var$/$var$_$year$$month$.nc'
          } else {
            stop('Only tas_anomaly (tas-tos) for HadCRUT4')
          }
          
          data = Start(dataset = path_obs,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'tasanomaly'){
            variable = 'tas-tos'
          }
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'HadSLP2'){
          
          if (variable == 'psl'){
            path_obs <- '/esarchive/obs/ukmo/hadslp_v2/monthly_mean/$var$/$var$_$year$$month$.nc'
          } else {stop('Incorrect variable')}
          
          data <- Start(dataset = path_obs,
                        var = variable,
                        year = paste0(years_obs),
                        month = 'all',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call, retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'ERA5'){
          
          if (identical(variable,'pr')){variable <- 'prlr'}
          
          if (variable %in% c('tas','psl','prlr')){
            path_obs <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
          } else {stop('Incorrect variable')}
          
          data <- Start(dataset = path_obs,
                        var = variable,
                        year = paste0(years_obs),
                        month = 'all',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call, retrieve = retrieve)
          
          if (identical(variable,'prlr')){variable <- 'pr'}
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset %in% c('EC-Earth3-i1_hist','EC-Earth3-i2_hist')){
          
          if (variable %in% c('tas','psl','pr','tos')){
            path_exp = '/esarchive/exp/ecearth/$member$/cmorfiles/CMIP/EC-Earth-Consortium/EC-Earth3/historical/$aux$/Amon/$var$/gr/$aux2$/$var$_Amon_EC-Earth3_historical_$aux$_gr_$year$01-$year$12.nc'
          } else {
            stop('Incorrect variable')
          }
          
          exps = c('a1wk','a1tt','a1tk','a1tn','a1to','a1ts','a247','a1tp','a1tr','a1ty')
          
          data = Start(dataset = path_exp,
                       var = variable,
                       year = as.character(years_hist),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       member = exps,
                       aux = 'all',
                       aux2 = 'all',
                       aux_depends = 'member',
                       aux_depends = 'aux2',
                       aux2_depends = 'member',
                       aux2_depends = 'aux',
                       synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude'), month = c('month','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'HadGEM3_hist'){
          
          if (variable == 'tos'){variable = 'ts'}
          
          if (variable %in% c('tas','psl')){
            # path = '/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-historical_i0p1/$var$_Amon_HadGEM3_historical_$member$_gr_$year$01-$year$12.nc'
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-historical_i0p1/CMIP/MOHC/HadGEM3-GC31-MM/historical/r$member$i1p1f1/Amon/$var$/gr/v20200101/$var$_Amon_HadGEM3_historical_r$member$i1p1f1_gr_$year$01-$year$12.nc'
          } else if (variable == 'ts'){
            path_exp <- '/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-historical_i0p1/CMIP/MOHC/HadGEM3-GC31-MM/historical/r$member$i1p1f1/Amon/$var$/gr/v20200101/$var$_Amon_HadGEM3-GC31-MM_historical_r$member$i1p1f1_gr_$year$01-$year$12.nc'
          }
          
          data = Start(dataset = path_exp,
                       var = variable,
                       year = as.character(years_hist),
                       month = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       member = members,
                       synonims = list(month = c('month','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'ts'){variable = 'tos'}
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'MPI-ESM1.2-HR_hist'){
          
          # ## the historical runs for the MPI-ESM1.2-HR are stored in files with 5 yeras
          # files_aux_monthly = paste0(seq(1965,2010,5),'01-',seq(1969,2014,5),'12')
          
          if (identical(years_hist, 1961:2014)){
            files_aux_monthly = paste0(seq(1960,2010,by = 5),'01-',seq(1964,2014,by = 5),'12')
          } else {stop('Check the period')}
          
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/historical/mpi-esm1-2-hr/cmip6-historical_i0p1/$var$_Amon_MPI-ESM1-2-HR_historical_$member$_gn_$aux$.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/historical/mpi-esm1-2-hr/cmip6-historical_i0p1/$var$_Omon_MPI-ESM1-2-HR_historical_$member$_gr_$aux$.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data = Start(dataset = path_exp,
                       var = variable,
                       aux = files_aux_monthly,
                       time = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       member = members,
                       synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.numeric(attr(data,'Variable')$dat1$lon)
          lat = as.numeric(attr(data,'Variable')$dat1$lat)
          
        } else if (dataset == 'CMCC-CM2-SR5_hist'){
          
          # only one file from 1850-01 to 2014-12 (for monthly tas, tos, pr and psl)
          if (identical(years_hist, 1965:2014)){
            times_aux_monthly = 1381:1980 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            times_aux_monthly <- 1333:1980 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          retrieve = TRUE # only one member
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/historical/cmcc-cm2-sr5/cmip6-historical_i0p1/$var$_Amon_*_historical_$member$_gn_185001-201412.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/historical/cmcc-cm2-sr5/cmip6-historical_i0p1/$var$_Omon_*_historical_$member$_gr_185001-201412.nc'
          } else {
            stop('Incorrect variable')
          }
          
          if (lat_min == -90 & lat_max == 90){lats = 'all'}
          if (lon_min == 0 & lon_max == 359.9){lons = 'all'}
          
          data = Start(dataset = path_exp,
                       var = variable,
                       month = times_aux_monthly,
                       lat = lats,#values(list(lat_min, lat_max)),
                       lon = lons,#values(list(lon_min, lon_max)),
                       member = members,
                       synonims = list(month = c('month','time'), lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'BCC-CSM2-MR_hist'){
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/historical/bcc-csm2-mr/cmip6-historical_i0p1/CMIP/BCC/BCC-CSM2-MR/historical/$member$/Amon/$var$/gn/v20200101/$var$_Amon_BCC-CSM2-MR_historical_$member$_gn_185001-201412.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/CMIP6/historical/bcc-csm2-mr/cmip6-historical_i0p1/CMIP/BCC/BCC-CSM2-MR/historical/$member$/Omon/$var$/gr/v20200101/$var$_Omon_BCC-CSM2-MR_historical_$member$_gr_185001-201412.nc'
          } else {stop('Incorrect variable')}
          
          if (identical(years_hist, 1965:2014)){
            times_aux = 1381:1980 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            times_aux <- 1333:1980 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        time = times_aux,
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members,
                        synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          # if(!identical(years_hist,1965:2014)){stop('Check the times for the historical years')}
          # 
          # if (variable %in% c('psl','pr','tos')){
          #   
          #   times_aux <- 1381:1980 # years 1965-2014
          #   path_exp <- ifelse(test = variable == 'tos', 
          #                      yes = '/esarchive/exp/CMIP6/historical/bcc-csm2-mr/cmip6-historical_i0p1/CMIP/BCC/BCC-CSM2-MR/historical/$member$/Omon/$var$/gr/v20200101/$var$_Omon_BCC-CSM2-MR_historical_$member$_gr_185001-201412.nc', 
          #                      no =  '/esarchive/exp/CMIP6/historical/bcc-csm2-mr/cmip6-historical_i0p1/CMIP/BCC/BCC-CSM2-MR/historical/$member$/Amon/$var$/gn/v20200101/$var$_Amon_BCC-CSM2-MR_historical_$member$_gn_185001-201412.nc')
          #   
          #   data <- Start(dataset = path_exp,
          #                 var = variable,
          #                 time = times_aux,
          #                 lat = values(list(lat_min, lat_max)),
          #                 lon = values(list(lon_min, lon_max)),
          #                 member = members,
          #                 synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
          #                 return_vars = list(lat = 'dataset', lon = 'dataset'),
          #                 lat_reorder = Sort(decreasing = F),
          #                 lon_reorder = CircularSort(0, 360),
          #                 num_procs = num_procs_start_call , retrieve = retrieve)
          #   
          # } else if (variable == 'tas'){
          #   
          #   freq <- 'daily'
          #   files_aux <- c('19500101-19741231','19750101-19991231','20000101-20141231')
          #   path_exp <- '/esarchive/exp/CMIP6/historical/bcc-csm2-mr/cmip6-historical_i0p1/CMIP/BCC/BCC-CSM2-MR/historical/$member$/day/$var$/gn/v20200101/$var$_day_BCC-CSM2-MR_historical_$member$_gn_$aux$.nc'
          #   
          #   data <- Start(dataset = path_exp,
          #                 var = variable,
          #                 time = 'all',
          #                 aux = files_aux,
          #                 lat = values(list(lat_min, lat_max)),
          #                 lon = values(list(lon_min, lon_max)),
          #                 member = members,
          #                 return_vars = list(lat = 'dataset', lon = 'dataset'),
          #                 lat_reorder = Sort(decreasing = F),
          #                 lon_reorder = CircularSort(0, 360),
          #                 num_procs = num_procs_start_call , retrieve = retrieve)
          #   
          # } else {stop('Incorrect variable')}
          
          lon = as.numeric(attr(data,'Variable')$dat1$lon)
          lat = as.numeric(attr(data,'Variable')$dat1$lat)
          
        } else if (dataset == 'CanESM5_hist'){
          
          if (identical(years_hist, 1965:2014)){
            times_aux = 1381:1980 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            times_aux <- 1333:1980 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/historical/canesm5/cmip6-historical_i0p2/CMIP/CCCma/CanESM5/historical/$member$/Amon/$var$/gn/v20190429/$var$_Amon_*_historical_$member$_gn_185001-201412.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/historical/canesm5/cmip6-historical_i0p2/CMIP/CCCma/CanESM5/historical/$member$/Omon/$var$/gr/v20190429/$var$_Omon_*_historical_$member$_gr_185001-201412.nc'
          } else {
            stop('Incorrect variable')
          }
          
          if (lat_min == -90 & lat_max == 90){lats = 'all'}
          if (lon_min == 0 & lon_max == 359.9){lons = 'all'}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        month = times_aux,
                        lat = lats,#values(list(lat_min, lat_max)),
                        lon = lons,#values(list(lon_min, lon_max)),
                        member = members,
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        synonims = list(month = c('month','time')),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.numeric(attr(data,'Variable')$dat1$lon)
          lat = as.numeric(attr(data,'Variable')$dat1$lat)
          
        } else if (dataset == 'CESM1-1-CAM5-CMIP5_hist'){
          
          ## NCAR LENS
          ## The files are stored as if they were decadal predictions init in Nov.
          ## Then January of year Y is the time step 3 of the file Y-1
          
          if (variable == 'pr'){variable <- 'prlr'}
          
          times_aux <- 3:14 # Jan to Dec
          
          if (variable %in% c('tas','psl','prlr')){
            path_exp <- '/esarchive/exp/ncar/lens/monthly_mean/$var$/$var$_$year$1101.nc'
          } else if (variable == 'tos'){
            path_exp <- '/esarchive/exp/ncar/lens/monthly_mean/$var$-lonlat/$var$_$year$1101.nc'
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        year = paste0(years_hist-1),
                        month = times_aux,
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members,
                        synonims = list(member = c('member', 'ensemble'), month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          if (variable == 'prlr'){variable <- 'pr'}
          
          lon = as.numeric(attr(data,'Variable')$dat1$lon)
          lat = as.numeric(attr(data,'Variable')$dat1$lat)
          
        } else if (dataset == 'IPSL-CM6A-LR_hist'){
          
          if (identical(years_hist, 1965:2014)){
            time_aux = 1381:1980 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            time_aux <- 1333:1980 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/historical/ipsl-cm6a-lr/cmip6-historical_i0p1/CMIP/IPSL/IPSL-CM6A-LR/historical/$member$/Amon/$var$/gr/v20200101/$var$_Amon_*_historical_$member$_gr_185001-201412.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/historical/ipsl-cm6a-lr/cmip6-historical_i0p1/CMIP/IPSL/IPSL-CM6A-LR/historical/$member$/Omon/$var$/gr/v20200101/$var$_Omon_*_historical_$member$_gr_185001-201412.nc'
          } else {
            stop('Incorrect variable')
          }
          
          if (lat_min == -90 & lat_max == 90){lats = 'all'}
          if (lon_min == 0 & lon_max == 359.9){lons = 'all'}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        month = time_aux,
                        lat = lats,#values(list(lat_min, lat_max)),
                        lon = lons,#values(list(lon_min, lon_max)),
                        member = members,
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        synonims = list(month = c('month','time')),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.numeric(attr(data,'Variable')$dat1$lon)
          lat = as.numeric(attr(data,'Variable')$dat1$lat)
          
        } else if (dataset == 'MIROC6_hist'){
          
          if (identical(years_hist, 1965:2014)){
            times_aux_monthly <- 181:780 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            times_aux_monthly <- 133:780 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          if (members == 'all'){members_aux = paste0('r',1:10,'i1p1f1')}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/historical/miroc6/cmip6-historical_i0p1/CMIP/MIROC/MIROC6/historical/$member$/Amon/$var$/gn/$version$/$var$_Amon_*_historical_$member$_gn_195001-201412.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/historical/miroc6/cmip6-historical_i0p1/CMIP/MIROC/MIROC6/historical/$member$/Omon/$var$/gr/$version$/$var$_Omon_*_historical_$member$_gr_195001-201412.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        month = times_aux_monthly,
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members_aux,
                        version = 'all',
                        version_depends = 'member',
                        synonims = list(month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'NorCPM1_hist'){
          
          if (identical(years_hist, 1965:2014)){
            time_aux = 1381:1980 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            time_aux <- 1333:1980 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/norcpm1/cmip6-historical_i0p1/original_files/cmorfiles/$var$_Amon_*_historical_$member$_gn_185001-201412.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/norcpm1/cmip6-historical_i0p1/original_files/cmorfiles/$var$_Omon_*_historical_$member$_gr_185001-201412.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        month = time_aux,
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members,
                        synonims = list(month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'MRI-ESM2-0_hist'){
          
          if (identical(years_hist, 1965:2014)){
            time_aux = 1381:1980 # (from 1965-01 to 2014-12)
          } else if (identical(years_hist, 1961:2014)){
            time_aux <- 1333:1980 # (from 1961-01 to 2014-12)
          } else {stop('check the period')}
          
          ## i1p1f1 added to the path because there are other two members with different "i" (r1i100 and r1i2)
          if (variable %in% c('tas','psl','pr')){
            path_exp = '/esarchive/exp/CMIP6/historical/mri-esm2-0/cmip6-historical_i0p1/CMIP/MRI/MRI-ESM2-0/historical/$member$i1p1f1/Amon/$var$/gn/v20200101/$var$_Amon_MRI-ESM2-0_historical_$member$i1p1f1_gn_185001-201412.nc'
          } else if (variable == 'tos'){
            path_exp = '/esarchive/exp/CMIP6/historical/mri-esm2-0/cmip6-historical_i0p1/CMIP/MRI/MRI-ESM2-0/historical/$member$i1p1f1/Omon/$var$/gr/v20200101/$var$_Omon_MRI-ESM2-0_historical_$member$i1p1f1_gr_185001-201412.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        month = time_aux,
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members,
                        synonims = list(month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'MPI-ESM1.2-LR_hist'){
          
          if (identical(members,'all')){members <- paste0('r',1:10,'i1p1f1')}
          
          if (variable %in% c('tas','psl','pr')){
            path_exp <- '/esarchive/exp/CMIP6/historical/mpi-esm1-2-lr/cmip6-historical_i0p1/CMIP/MPI-M/MPI-ESM1-2-LR/historical/$member$/Amon/$var$/gn/v20200101/$var$_Amon_MPI-ESM1-2-LR_historical_$member$_gn_$aux$.nc'
          } else if (identical(variable,'tos')){
            path_exp <- '/esarchive/scratch/cdelgado/data/MPI-ESM1.2-LR_tos_gr/$var$_Omon_MPI-ESM1-2-LR_historical_$member$_gn_$aux$.nc'
          } else {stop('Incorrect variable')}
          
          if (identical(years_hist, 1961:2014)){
            files_aux <- c('195001-196912', '197001-198912', '199001-200912', '201001-201412')
          } else {stop('check the period, and edit load_average_monthly.R')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        month = 'all',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members,
                        aux = files_aux,
                        month_across = 'aux',
                        merge_across_dims = TRUE,
                        synonims = list(month = c('month','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else {stop('Incorrect dataset')}
        
      } else if (variable %in% c('tasmin','tasmax')){
        
        freq <- 'daily'
        
        ################################################################################################
        ############################## LOAD THE DAILY DATA #############################################
        ################################################################################################
        
        if (dataset == 'EC-Earth3-i1'){
          
          # added another fyear for the last 2 months (init in N)
          fyears_ECEarth = fyears[1] : (fyears[length(fyears)]+1)
          
          path <- '/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/day/$var$/gr/v20190713/$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$fyear$.nc'
          
          data <- Start(dataset = path,
                        var = variable,
                        sdate = paste0(sdates),
                        fday = 'all',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fyear = indices(fyears_ECEarth),
                        member = members,
                        fyear_depends = 'sdate',
                        fday_across = 'fyear',
                        merge_across_dims = TRUE,
                        synonims = list(fday = c('fday','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'HadGEM3'){
          
          fdays <- get_days_dcpp(fyears = fyears, season = season, init = 'Nov', calendar = '360-days')
          path <- '/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/$member$/day/$var$/gn/v20200101/$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-$member$_gn_$sdate$1101-$aux$1230.nc'
          
          data <- Start(dataset = path,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fday = indices(fdays),
                        member = members,
                        synonims = list(fday = c('fday','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1[['lon']])
          lat = as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'MPI-ESM1.2-HR'){
          
          fdays <- get_days_dcpp(fyears = fyears, season = season, init = 'Nov', calendar = '365-days')
          path <- '/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/dcppA-hindcast/$member$/day/$var$/gn/v20200101/$var$_*_s$sdate$-$member$_gn_$sdate$1101-$aux$1231.nc'
          
          data <- Start(dataset = path,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fday = indices(fdays),
                        member = members,
                        synonims = list(fday = c('fday','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'CMCC-CM2-SR5'){
          
          fdays <- get_days_dcpp(fyears = fyears, season = season, init = 'Nov', calendar = '365-days')
          
          path <- '/esarchive/exp/CMIP6/dcppA-hindcast/cmcc-cm2-sr5/cmip6-dcppA-hindcast_i1p1/DCPP/CMCC/CMCC-CM2-SR5/dcppA-hindcast/$member$/day/$var$/gn/v20200101/$var$_day_CMCC-CM2-SR5_dcppA-hindcast_s$sdate$-$member$_gn_$sdate$1101-$aux$1231.nc'
          
          data <- Start(dataset = path,
                        var = variable,
                        sdate = paste0(sdates),
                        aux = 'all',
                        aux_depends = 'sdate',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        fday = indices(fdays),
                        member = members,
                        synonims = list(fday = c('fday','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1[['lon']])
          lat <- as.vector(attr(data,'Variables')$dat1[['lat']])
          
        } else if (dataset == 'EC-Earth3_hist'){
          
          path_exp <- '/esarchive/exp/ecearth/$member$/cmorfiles/CMIP/EC-Earth-Consortium/EC-Earth3/historical/$aux$/day/$var$/gr/$aux2$/$var$_day_EC-Earth3_historical_$aux$_gr_$year$0101-$year$1231.nc'
          exps <- c('a1wk','a1tt','a1tk','a1tn','a1to','a1ts','a247','a1tp','a1tr','a1ty')
          
          if (season == 'Annual'){
            days <- 'all'
          } else if (season == 'JJA'){
            days <- indices(152:243)
          } else if (season == 'DJF'){
            days <- indices(c(1:59,335:365))
          } else {stop('Incorrect season')}
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        year = as.character(years_hist),
                        day = days,
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        aux = 'all',
                        aux2 = 'all',
                        member = exps,
                        synonims = list(day = c('day', 'time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call, retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'HadGEM3_hist'){
          
          if (variable %in% c('tasmin','tasmax')){
            path_exp = '/esarchive/exp/CMIP6/historical/hadgem3-gc31-mm/cmip6-historical_i0p1/$var$_day_HadGEM3-GC31-MM_historical_$member$f2_gn_$year$0101-$year$1230.nc'
          } else {
            stop('Incorrect variable')
          }
          
          if (season == 'Annual'){ # 360-days calendar
            days <- 'all'
          } else if (season == 'JJA'){
            days <- indices(151:240)
          } else if (season == 'DJF'){
            days <- indices(c(1:60,331:360))
          } else {stop('Incorrect season')}
          
          data = Start(dataset = path_exp,
                       var = variable,
                       year = as.character(years_hist),
                       day = days,
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       member = members,
                       synonims = list(day = c('time','day')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.vector(attr(data,'Variables')$dat1$lon)
          lat = as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'MPI-ESM1.2-HR_hist'){
          
          # the historical runs for the MPI-ESM1.2-HR are stored in files with 5 yeras
          files_aux_daily = paste0(seq(1965,2010,5),'0101-',seq(1969,2014,5),'1231')
          
          if (variable %in% c('tasmin','tasmax')){
            path_exp = '/esarchive/exp/CMIP6/historical/mpi-esm1-2-hr/cmip6-historical_i0p1/$var$_day_MPI-ESM1-2-HR_historical_$member$_gn_$aux$.nc'
          } else {
            stop('Incorrect variable')
          }
          
          data = Start(dataset = path_exp,
                       var = variable,
                       aux = files_aux_daily,
                       day = 'all',
                       lat = values(list(lat_min, lat_max)),
                       lon = values(list(lon_min, lon_max)),
                       member = members,
                       synonims = list(day = c('day','time')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       lat_reorder = Sort(decreasing = F),
                       lon_reorder = CircularSort(0, 360),
                       num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon = as.numeric(attr(data,'Variable')$dat1$lon)
          lat = as.numeric(attr(data,'Variable')$dat1$lat)
          
        } else if (dataset == 'CMCC-CM2-SR5_hist'){
          
          times_aux_daily <- c('19500101-19741231','19750101-19991231','20000101-20141231')
          
          if (variable %in% c('tasmin','tasmax')){
            path_exp <- '/esarchive/exp/CMIP6/historical/cmcc-cm2-sr5/cmip6-historical_i0p1/$var$_day_*_historical_$member$_gn_$aux$.nc'
          } else {
            stop('Incorrect variable')
          }
          # data stored in files with 25 years (except the last one with data from 2000 to 2014)
          
          data <- Start(dataset = path_exp,
                        var = variable,
                        aux = times_aux_daily,
                        day = 'all',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        member = members,
                        synonims = list(day = c('day','time')),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call , retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1$lon)
          lat <- as.vector(attr(data,'Variables')$dat1$lat)
          
        } else if (dataset == 'JRA-55'){
          
          retrieve <- FALSE
          
          ## monthly mean from f6h in esarchive
          path <- '/esarchive/recon/jma/jra55/daily/$var$-288x145_f6h/$var$_$year$$month$.nc'
          
          data <- Start(dataset = path,
                        var = variable,
                        year = paste0(years_obs),
                        month = 'all',
                        time = 'all',
                        lat = values(list(lat_min, lat_max)),
                        lon = values(list(lon_min, lon_max)),
                        return_vars = list(lat = 'dataset', lon = 'dataset'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0, 360),
                        num_procs = num_procs_start_call, retrieve = retrieve)
          
          lon <- as.vector(attr(data,'Variables')$dat1$lon)
          lat <- as.vector(attr(data,'Variables')$dat1$lat)
          
        } else {stop('Incorrect dataset')}
        
      } else {stop('Incorrect variable')}
      
      ################################################################################################
      ############################################### CHECKING #######################################
      ################################################################################################
      
      if (!is.null(attr(data, 'NotFoundFiles'))){
        stop('There are not found files. Check the pattern path or if any file is missing!')
      }
      
      if (retrieve){
        print(dim(data))
      } else {
        print(attr(data, 'Dimensions'))
      }
      
      ################################################################################################
      ################################# LOADING AND AVERAGING ########################################
      ################################################################################################
      
      if (freq == 'monthly'){
        load_and_average <- load_and_average_monthly
      } else if (freq == 'daily'){
        load_and_average <- load_and_average_daily
      }
      
      if (retrieve == TRUE){
        
        output <- load_and_average(data = data, dataset = dataset, season = season, n_cores = ncores)
        
      } else if (retrieve == FALSE){
        
        if (dataset %in% c(models_dcpp,models_WMO_dcpp)){
          
          if (freq == 'monthly'){
            target_dims = c('dataset','fmonth')
          } else if (freq == 'daily'){
            target_dims = c('dataset','fday')
          }
          output_dims = c('dataset')
          
          if (dataset == 'CESM1-1-CAM5-CMIP5'){
            chunks <- list(member = round(as.numeric(attr(data,'Dimensions')['member'])/20,0), 
                           sdate = round(as.numeric(attr(data,'Dimensions')['sdate'])/2,0))
          } else {
            chunks <- list(sdate = as.numeric(attr(data,'Dimensions')['sdate']))
          }
          
        } else if (dataset %in% c(models_hist,models_WMO_hist)){
          
          if (freq == 'monthly'){
            
            if (dataset %in% c('MPI-ESM1.2-HR_hist')){
              target_dims = c('aux','time')
              output_dims = c('year')
            } else if (dataset == 'BCC-CSM2-MR_hist'){
              stop('run with retrieve = T')
              target_dims <- c('time')
              output_dims <- c('year')
            } else if (dataset %in% c('MIROC6_hist','IPSL-CM6A-LR_hist','NorCPM1_hist','CanESM5_hist','MRI-ESM2-0_hist','MPI-ESM1.2-LR_hist')){
              target_dims <- c('month')
              output_dims <- c('year')
            } else {
              target_dims = c('month','year','dataset','var')
              output_dims = c('year','dataset','var')
            }
            
          } else if (freq == 'daily'){
            
            if (dataset %in% c('EC-Earth3_hist','HadGEM3_hist')){
              target_dims = c('day','year')
            } else if (dataset %in% c('MPI-ESM1.2-HR_hist','CMCC-CM2-SR5_hist')){
              target_dims = c('aux','day')
            } else if (dataset == 'BCC-CSM2-MR_hist'){
              target_dims <- c('time','aux')
            }
            output_dims = c('year')
          }
          
          if (dataset == 'BCC-CSM2-MR_hist'){
            chunks <- list(member = as.numeric(attr(data,'Dimensions')['member']),
                           lat = round(as.numeric(attr(data,'Dimensions')['lat'])/10,0))
          } else {
            chunks <- list(member = as.numeric(attr(data,'Dimensions')['member']))
          }
          
        } else if (dataset %in% c(reanalyses,observations)){
          
          target_dims <- c('year','month','time')
          output_dims <- c('year')
          chunks <- list(lat = round(as.numeric(attr(data,'Dimensions')['lat'])/5,0))
          
        }
        
        step <- startR::Step(fun = load_and_average,
                             use_libraries = c('startR','multiApply','s2dv','CSTools'),
                             target_dims = target_dims, 
                             output_dims = output_dims)
        
        wf <- startR::AddStep(inputs = data, 
                              step_fun = step,
                              dataset = dataset,
                              season = season,
                              n_cores = ncores)
        
        output <- startR::Compute(workflow = wf,
                                  chunks = chunks,
                                  threads_load = num_procs_start_call,
                                  threads_compute = ncores,
                                  cluster = list(queue_host = 'nord1.bsc.es',
                                                 queue_type = 'lsf',
                                                 temp_dir = '/gpfs/scratch/bsc32/bsc32924/startR_hpc/',
                                                 cores_per_job = ncores,
                                                 job_wallclock = '05:00',
                                                 max_jobs = max_jobs,
                                                 extra_queue_params = list('#BSUB -q bsc_es -M 5000'), # node exclusive: #BSUB -x
                                                 bidirectional = FALSE,
                                                 polling_period = 10),
                                  ecflow_suite_dir = '/home/Earth/cdelgado/Desktop/startR_local/',
                                  wait = TRUE)$output1
      }
      
      ################################################################################################
      ########################################## SAVE THE RESULTS ####################################
      ################################################################################################
      
      if (season == 'DJF'){
        years_obs = years_obs[2:length(years_obs)]
        years_hist = years_hist[2:length(years_hist)]
      }
      
      X <- NULL
      X$data <- drop(output)
      X$lat <- lat
      X$lon <- lon
      X$season <- season
      
      if (dataset %in% c(models_dcpp,models_WMO_dcpp)){
        if (dataset == 'BCC-CSM2-MR'){
          X$sdates <- sdates - 1
          X$real_sdates <- sdates
        } else {
          X$sdates <- sdates
        }
        stopifnot(length(X$sdates) == as.numeric(dim(X$data)['sdate']))
      } else if (dataset %in% c(models_hist,models_WMO_hist)){
        X$years <- years_hist
        stopifnot(length(X$years) == as.numeric(dim(X$data)['year']))
      } else if (dataset %in% c(reanalyses,observations)){
        X$years <- years_obs
        stopifnot(length(X$years) == as.numeric(dim(X$data)['year']))
      } else {
        stop('dataset not in c(models_dcpp, models_hist, observations, reanalyses)')
      }
      
      ## The data is saved with the following units:
      ## - tas: K
      ## - psl: Pa
      ## - pr: kg m-2 s-1
      ## - tos: K
      ## - tasmax: K
      ## - tasmin: K
      
      ## Changing units to have the same ones for all the datasets
      if (dataset %in% c('GHCNv4','GISTEMPv4','HadCRUT4')){
        X <- change_clim_ref_period(X = X, dataset = dataset, clim_first_year = 1981, clim_end_year = 2010)
        X$data <- X$data + 273.15
      } else if (dataset %in% c('GPCC','CESM1-1-CAM5-CMIP5','CESM1-1-CAM5-CMIP5_hist','ERA5') & variable == 'pr'){
        X$data <- X$data * 1000
      } else if (dataset %in% c('HadGEM3') & variable == 'pr'){
        X$data = X$data / (24*60*60)
      } else if (dataset == 'CMCC-CM2-SR5_hist'){
        # to insert the member dimension (it has only one member, and the dimension is then dropped before)
        if (variable == 'tos'){
          X$data = X$data + 273.15
        }
        X$data = s2dv::InsertDim(data = X$data, posdim = 4, lendim = 1, name = 'member', ncores = 4)
      } else if (dataset %in% c('CMCC-CM2-SR5','MPI-ESM1.2-HR','MPI-ESM1.2-HR_hist','BCC-CSM2-MR','BCC-CSM2-MR_hist','CanESM5','IPSL-CM6A-LR','IPSL-CM6A-LR_hist','MIROC6','MIROC6_hist','NorCPM1','NorCPM1_hist','CanESM5_hist','MRI-ESM2-0','MRI-ESM2-0_hist','EC-Earth3-i2') & variable == 'tos'){
        X$data = X$data + 273.15
      } 
      
      print(str(X))
      print(summary(X$data))
      
      if (dataset %in% c(models_dcpp, models_WMO_dcpp)){
        if (length(fyears)==1){
          saveRDS(X, file = paste0(outputs_path, variable, '_', dataset, '_fy', fyears, '_',season, '.rds'))
          print(paste0('Finished ', dataset, ' - ', variable, ' - ', season, ' - fy', fyears))
        } else {
          saveRDS(X, file = paste0(outputs_path, variable, '_', dataset, '_fy', fyears[1],'-',fyears[length(fyears)], '_',season, '.rds'))
          print(paste0('Finished ', dataset, ' - ', variable, ' - ', season, ' - fy', fyears[1],'-',fyears[length(fyears)]))
        }
      } else if (dataset %in% c(models_hist, models_WMO_hist, observations, reanalyses)) {
        saveRDS(X, file = paste0(outputs_path, variable, '_', dataset, '_',season, '.rds'))
        print(paste0('Finished ', dataset, ' - ', variable, ' - ', season))
      }
    }
  }
}

