# https://earth.bsc.es/gitlab/es/csdownscale/-/issues/2

library(s2dv)
library(startR)
library(lubridate)
library(ncdf4)

source('/home/Earth/aho/aho-testtest/s2dv/CDORemap_downscaling_Jaume.R')
.KnownLonNames <- s2dv:::.KnownLonNames
.KnownLatNames <- s2dv:::.KnownLatNames

target_grid <- '/esarchive/recon/ecmwf/era5/monthly_mean/tas_f1h/tas_200002.nc'
method <- 'con'
lonmin <- -11.5 - 1
lonmax <- 5.35 + 1
latmin <- 35.1 - 1
latmax <- 44.1 + 1
sdates <- c('20000201')#,'20010201','20020201','20030201','20040201','20050201','20060201','20070201')

# Load high-res observations
obs <- Start(dat = '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
             var = 'tas', time = indices(1), lat = values(list(latmin, latmax)),
             lat_reorder = Sort(decreasing = T), lon = values(list(lonmin, lonmax)),
             lon_reorder = CircularSort(-180, 180), sdate = format(ymd(sdates), "%Y%m"),
             synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'),
             lat = c('lat', 'latitude')), return_vars = list(lat = 'dat', lon = 'dat'),
             retrieve = TRUE)

# Load coarse model data
exp <- Start(dat = '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas', time = indices(1), member = indices(1), sdate = sdates,
             lat = values(list(latmin, latmax)), lat_reorder = Sort(decreasing = T),
             lon = values(list(lonmin, lonmax)), lon_reorder = CircularSort(-180, 180),
             synonims = list(var = c('var','variable'), lon = c('lon', 'longitude'),
             lat = c('lat', 'latitude'), member = c('member','ensemble')),
             return_vars = list(lat = 'dat', lon = 'dat'),
             retrieve = TRUE)

exp <- drop(exp)
obs <- drop(obs)
dim(exp)
#lat lon 
#  9  17 

dim(obs)
#lat lon 
# 32  60 

crop_ref <- obs
west <- attr(crop_ref, "Variables")$dat1$lon[1]
east <- attr(crop_ref, "Variables")$dat1$lon[length(attr(crop_ref, "Variables")$dat1$lon)]
south <- attr(crop_ref, "Variables")$dat1$lat[1]
north <- attr(crop_ref, "Variables")$dat1$lat[length(attr(crop_ref, "Variables")$dat1$lat)]

# Regrid with CDORemap
exp_interpolated <- CDORemap(data_array = exp,
                             lats = attr(exp, "Variables")$dat1$lat,
                             lons = attr(exp, "Variables")$dat1$lon,
                             grid = target_grid,
                             method = method,
                             crop = c(west, east, south, north),
                             write_dir = '/home/Earth/aho/')

# Compare the dimensions of the interpolated model data and the high-res observations
dim(obs)
dim(exp_interpolated$data)

lons_obs <- as.vector(attr(obs, 'Variables')$dat1$lon)
lats_obs <- as.vector(attr(obs, 'Variables')$dat1$lat)
as.vector(exp_interpolated$lat)


tmp_file <- "/home/Earth/aho/R_CDORemap_input_exp.nc"
tmp_file2 <- "/home/Earth/aho/R_CDORemap_output_exp.nc"

sellonlatbox <- paste0('sellonlatbox,', format(west, scientific = FALSE), ',', format(east, scientific = FALSE), ',', format(south, scientific = FALSE), ',', format(north, scientific = FALSE), ' -')

paste0("cdo -s ", sellonlatbox, "remap", method, ",", target_grid, " ", tmp_file, " ", tmp_file2)
