#Clear workspace
rm(list = ls())

#library(maps)
#library(boot)
#library(RColorBrewer)
#library(easyVerification)
#library(SpecsVerification)
library(s2dv)
#library(abind)
#library(multiApply)
#library(ncdf4)
library(startR)
#library(plyr)

retrieve <- FALSE

exp <- 'a3w5'
vari <- 'tas'
mod <- 'Amon'
grid <- '1x1'
#NOTE: era5 from 197901 to 202006
yr1 <- '1979'
yr2 <- '2018'
t1 <- 1
t2 <- 14 # for first whole year (JAN to DEC) 
t2 <- 2 + (12 * 5) # fyear 1:5
chosen_season <- 'Annual' #'DJF'
lats <- c(-90, 90)
lons <- c(0, 359.9)
#PROBLEM: a3w5 should have i4, not i1; but grid '1x1' is only under i1
membs <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------

repos <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/',
                mod, '/$var$/', grid, '/v*/$var$_', mod,
                '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid,
                '_$chunk$.nc')

data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = paste(seq(yr1, yr2)),
              chunk = 'all',
              chunk_depends = 'sdate',
              time = c(t1:t2), #'all',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
#              latitude = 'all',
              latitude_reorder = Sort(),
#              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
#              transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = 'r360x180', method = 'con', crop = FALSE), transform_vars = c('lat','lon'),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)
attr(data, 'Dimensions')


#--------------------------
# Step2: Load obs
#--------------------------
obs <- 'era5'
path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h-r360x180/$var$_$file_date$.nc')
#path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h/$var$_$file_date$.nc')
#path_obs <- '/esarchive/obs/ukmo/hadisst_v1.1/monthly_mean/$var$/$var$_$file_date$.nc'
time_data <- attr(data, 'Variables')$common$time #[sdate, time]
dim(time_data)

#---------------------------------------------
#Change the version before bug being fixed.
detach("package:startR", unload = T)
require(startR, lib.loc = "/shared/earth/software/R/3.6.1-foss-2015a-bare/lib64/R/library") #change the loaded library
#---------------------------------------------

data_obs <- Start(dat = path_obs,
              var = vari,
              file_date = unique(format(time_data, '%Y%m')), #paste(seq(yr1, yr2)),
              time = values(time_data),
              time_across = 'file_date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
#              transform = CDORemapper, transform_extra_cells = 2,
#              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
#                                      method = 'con'),
#              transform_vars = c('latitude', 'longitude'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)
attr(data_obs, 'Dimensions')

# CHECK ATTRIBUTES
## dimensions
dim_data <- attr(data, 'Dimensions')
dim_obs <- attr(data_obs, 'Dimensions')
if (any(!names(dim_obs) %in% names(dim_data))) {
  stop("obs does not have the same dimension names as exp.")
} else if (any(dim_data[match(names(dim_obs), names(dim_data))] != dim_obs)) {
  stop("obs and exp do not have the same dimensions.")
}

## time
time_obs <- attr(data_obs, 'Variables')$common$time 
if (!all.equal(format(time_obs, '%Y%m'), format(time_data, '%Y%m'))) {
  stop("obs and exp do not have the same time (%Y%m).")
}

## lat and lon
lat_obs <- attr(data_obs, 'Variables')$common$latitude
lon_obs <- attr(data_obs, 'Variables')$common$longitude
lat_data <- attr(data, 'Variables')$common$latitude
lon_data <- attr(data, 'Variables')$common$longitude
if (!all.equal(lat_obs, lat_data , check.attributes = F) |
    !all.equal(lon_obs, lon_data, check.attributes = F)) {
  stop("obs and exp do not have the same lat/lon.")
}


#--------------------------
# Step3: Build workflow
#--------------------------
  source('~/aho-testtest/shiny/cp_shiny_decadal/fun_Season_diffClim_ACC.R')

  step <- Step(fun = fun_Season_diffClim_ACC,
               use_libraries = c('s2dv', 'ClimProjDiags'),
               target_dims = list(data = c('sdate', 'time', 'memb'),
                                  data_obs = c('sdate', 'time')),
               output_dims = list(diff = 'time', acc = 'time', acc_sig = 'time'))
  if (chosen_season == 'Annual') {
    chosen_mon <- c(1, 12)
  } else if (chosen_season == 'DJF') {
    chosen_mon <- c(12, 2)
  } else if (chosen_season == 'JJA') {
    chosen_mon <- c(6, 8)
  } else if (chosen_season == 'SON') {
    chosen_mon <- c(9, 11)
  } else {
    stop("Undefined season. Assign another chosen_season.")
  }
  wf <- AddStep(inputs = list(data, data_obs),
                step_fun = step, chosen_mon = chosen_mon)


#--------------------------
# Step4: Compute
#--------------------------

# local
  output <- Compute(workflow = wf$diff,
                    chunks = list(longitude = 4))
dim(output$diff)


# Nord3v2
#-------------------user-defined---------------------
  queue_host <- 'nord4'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------
  output <- Compute(wf$diff,
                 chunks = list(latitude = 8,
                               longitude = 8),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(
                   queue_host = queue_host,
                   queue_type = 'slurm',
                   temp_dir = temp_dir,
                   cores_per_job = 8,
                   job_wallclock = '01:00:00',
                   max_jobs = 16,
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = TRUE
                 )  #$output1


#--------------------------
# Step5: Plotting
#--------------------------

# Plotting
diff_clim_obs <- drop(output$diff)
acc_data <- drop(output$acc)
acc_sig <- drop(output$acc_sig)

lon <- as.vector(attr(data, 'Variables')$common$longitude)
lat <- as.vector(attr(data, 'Variables')$common$latitude)
chosen_yr <- 1 
#color_range <- c(-max(abs(range(diff_clim_obs[chosen_yr, , ]))), max(abs(range(diff_clim_obs[chosen_yr, , ]))))
color_range_diffClim <- c(-5, 5)

# Calculate fyr average (it's better to be done in function)
if ('time' %in% names(dim(acc_data))) {
fyr <- 2:5
fyr_chr <- paste0(fyr[1], '-', fyr[length(fyr)])
acc_data <- ClimProjDiags::Subset(acc_data, 'time', fyr, drop = 'selected')
acc_sig <- ClimProjDiags::Subset(acc_sig, 'time', fyr, drop = 'selected')
} else {
fyr <- 1
fyr_chr <- paste0(fyr[1])
}

# Plot
if (all(names(dim(diff_clim_obs)) %in% c('latitude', 'longitude'))) {
  # diff
  s2dv::PlotEquiMap(diff_clim_obs, lon = lon, lat = lat,
                    toptitle = paste0('bias (to ', obs, ') ', exp, ', ', 
                                       vari, ', fyr ', fyr_chr, ', ', chosen_season),
                    filled.continents = FALSE, bar_limits = color_range_diffClim, 
                    fileout = paste0('~/dcpp_map_diff_', chosen_season, '.png'))

  # acc
  s2dv::PlotEquiMap(acc_data, lon = lon, lat = lat,
                    toptitle = paste0('ACC (to ', obs, ') ', exp, ', ', 
                                       vari, ', fyr ', fyr_chr, ', ', chosen_season),
                    filled.continents = FALSE, bar_limits = c(-1, 1), 
                    dots = acc_sig,
                    fileout = paste0('~/dcpp_map_acc_', chosen_season, '.png'))

} else {
  # diff
s2dv::PlotEquiMap(diff_clim_obs[1, , ], lon = lon, lat = lat, 
                  toptitle = paste0('SST bias (to ', obs, ') ', exp, ', ', 
                                     vari, ', fyr', chosen_yr, ', ', chosen_season),
                  filled.continents = FALSE, bar_limits = color_range_diffClim, 
                  fileout = paste0('~/dcpp_map_diff_', chosen_season, '.png'))

