#Clear workspace
rm(list = ls())

#library(maps)
#library(boot)
#library(RColorBrewer)
#library(easyVerification)
#library(SpecsVerification)
library(s2dv)
#library(abind)
#library(multiApply)
#library(ncdf4)
library(startR)
#library(plyr)

retrieve <- FALSE

exp <- 'a3w5'
vari <- 'tas'
mod <- 'Amon'
grid <- '1x1'
#NOTE: era5 from 197901 to 202006
yr1 <- '1979'
yr2 <- '2018'
t1 <- 1
t2 <- 14 # for first 'DJF' 
chosen_season <- 'Annual' #'DJF'
lats <- c(-90, 90)
lons <- c(0, 359.9)
#PROBLEM: a3w5 should have i4, not i1; but grid '1x1' is only under i1
membs <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')
nmemb <- length(membs)

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------

repos <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/',
                mod, '/$var$/', grid, '/v*/$var$_', mod,
                '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid,
                '_$chunk$.nc')

data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = paste(seq(yr1, yr2)),
              chunk = 'all',
              chunk_depends = 'sdate',
              time = c(t1:t2), #'all',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
#              latitude = 'all',
              latitude_reorder = Sort(),
#              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
#              transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = 'r360x180', method = 'con', crop = FALSE), transform_vars = c('lat','lon'),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)
attr(data, 'Dimensions')


#--------------------------
# Step3: Load obs
#--------------------------
obs <- 'era5'
path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h-r360x180/$var$_$file_date$.nc')
#path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h/$var$_$file_date$.nc')
#path_obs <- '/esarchive/obs/ukmo/hadisst_v1.1/monthly_mean/$var$/$var$_$file_date$.nc'
time_data <- attr(data, 'Variables')$common$time #[sdate, time]

#---------------------------------------------
#Change the version before bug being fixed.
detach("package:startR", unload = T)
require(startR, lib.loc = "/shared/earth/software/R/3.6.1-foss-2015a-bare/lib64/R/library") #change the loaded library
#---------------------------------------------

data_obs <- Start(dat = path_obs,
              var = vari,
              file_date = unique(format(time_data, '%Y%m')), #paste(seq(yr1, yr2)),
              time = values(time_data),
              time_across = 'file_date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
#              transform = CDORemapper, transform_extra_cells = 2,
#              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
#                                      method = 'con'),
#              transform_vars = c('latitude', 'longitude'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'),
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)
attr(data_obs, 'Dimensions')

# CHECK ATTRIBUTES
## dimensions
dim_data <- attr(data, 'Dimensions')
dim_obs <- attr(data_obs, 'Dimensions')
if (any(!names(dim_obs) %in% names(dim_data))) {
  stop("obs does not have the same dimension names as exp.")
} else if (any(dim_data[match(names(dim_obs), names(dim_data))] != dim_obs)) {
  stop("obs and exp do not have the same dimensions.")
}

## time
time_obs <- attr(data_obs, 'Variables')$common$time 
if (!all.equal(format(time_obs, '%Y%m'), format(time_data, '%Y%m'))) {
  stop("obs and exp do not have the same time (%Y%m).")
}

## lat and lon
lat_obs <- attr(data_obs, 'Variables')$common$latitude
lon_obs <- attr(data_obs, 'Variables')$common$longitude
lat_data <- attr(data, 'Variables')$common$latitude
lon_data <- attr(data, 'Variables')$common$longitude
if (!all.equal(lat_obs, lat_data , check.attributes = F) |
    !all.equal(lon_obs, lon_data, check.attributes = F)) {
  stop("obs and exp do not have the same lat/lon.")
}


#--------------------------
# Step4: Build workflow
#--------------------------

 fun_Season_diffClim <- function(data, data_obs, chosen_mon) {
    # data: [sdate, time, memb]
    # data: [sdate, time]
    # chosen_mon: c(moninf, monsup)
#print('in fun!!')
    data_yrs <- apply(data, c(1, 3), s2dv:::.Season, monini = 11, moninf = chosen_mon[1], monsup = chosen_mon[2], method = mean, na.rm = TRUE)
    data_obs_yrs <- apply(data_obs, 1, s2dv:::.Season, monini = 11, moninf = chosen_mon[1], monsup = chosen_mon[2], method = mean, na.rm = TRUE)
    #data_yrs should be [time, sdate, memb], data_obs_yrs should be [time, sdate]
    if (length(dim(data_yrs)) != 3) {
      data_yrs <- array(data_yrs, dim = c(time = 1, dim(data_yrs)['sdate'], dim(data_yrs)['memb']))
    }
    if (length(dim(data_obs_yrs)) != 2) {
      data_obs_yrs <- array(data_obs_yrs, dim = c(time = 1, sdate = length(data_obs_yrs)))
    }

    data_yrs <- s2dv::InsertDim(data_yrs, 3, 1, 'dat')
    data_obs_yrs <- s2dv::InsertDim(s2dv::InsertDim(data_obs_yrs, 3, 1, 'dat'), 4, 1, 'memb')
    clim_data <- array(dim = c(time = dim(data_yrs)[1]))
    clim_data_obs <- array(dim = c(time = dim(data_yrs)[1]))
    for (tt in 1:dim(data_yrs)[1]) {
      data_tmp <- ClimProjDiags::Subset(data_yrs, 1, tt, drop = 'selected')
      data_obs_tmp <- ClimProjDiags::Subset(data_obs_yrs, 1, tt, drop = 'selected')
      tmp <- s2dv:::.Clim(data_tmp, data_obs_tmp,
                          time_dim = 'sdate', method = "clim", dat_dim = c('dat', 'memb'),
                          memb_dim = 'memb', memb = FALSE, na.rm = TRUE)
      clim_data[tt] <- tmp$clim_exp
      clim_data_obs[tt] <- tmp$clim_obs
    }

    # diff (clim - obs)
    diff <- clim_data - clim_data_obs
    return(diff)
#    return(list(clim_exp = clim_data, clim_obs = clim_data_obs))
  }


  step <- Step(fun = fun_Season_diffClim,
               use_libraries = c('s2dv', 'ClimProjDiags'),
               target_dims = list(data = c('sdate', 'time', 'memb'),
                                  data_obs = c('sdate', 'time')),
#               output_dims = list(clim_exp = c('time'), clim_obs = c('time')))
               output_dims = 'time')
  if (chosen_season == 'Annual') {
    chosen_mon <- c(1, 12)
  } else if (chosen_season == 'DJF') {
    chosen_mon <- c(12, 2)
  } else if (chosen_season == 'JJA') {
    chosen_mon <- c(6, 8)
  } else if (chosen_season == 'SON') {
    chosen_mon <- c(9, 11)
  } else {
    stop("Undefined season. Assign another chosen_season.")
  }
  wf <- AddStep(inputs = list(data, data_obs),
                step_fun = step, chosen_mon = chosen_mon)


#--------------------------
# Step5: Compute
#--------------------------

# local
  output <- Compute(workflow = wf, #wf$clim_exp,
                    chunks = list(longitude = 2))
dim(output$output1)


# Nord3v2
#-------------------user-defined---------------------
  queue_host <- 'nord4'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------
  output <- Compute(wf,
                 chunks = list(latitude = 4,
                               longitude = 4),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(
                   queue_host = queue_host,
                   queue_type = 'slurm',
                   temp_dir = temp_dir,
#                   lib_dir = "/.statelite/tmpfs/gpfs/home/bsc32/bsc32734/R/x86_64-pc-linux-gnu-library/3.6",
                   cores_per_job = 4,
                   job_wallclock = '01:00:00',
                   max_jobs = 4,
 #                  extra_queue_params = list('#BSUB -q bsc_es'),
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = TRUE
                 )  #$output1


#--------------------------
# Step6: Plotting
#--------------------------

# Plotting
diff_clim_obs <- drop(output$output1)
summary(diff_clim_obs)
lons <- as.vector(attr(data, 'Variables')$common$longitude)
lats <- as.vector(attr(data, 'Variables')$common$latitude)
chosen_yr <- 1 
#color_range <- c(-max(abs(range(diff_clim_obs[chosen_yr, , ]))), max(abs(range(diff_clim_obs[chosen_yr, , ]))))
color_range <- c(-5, 5)

if (all(names(dim(diff_clim_obs)) %in% c('latitude', 'longitude'))) {
  s2dv::PlotEquiMap(diff_clim_obs, lon = lons, lat = lats, 
                    toptitle = paste0('SST bias (to ', obs, ') ', exp, ', ', 
                                       vari, ', fyr 1, ', chosen_season),
                    filled.continents = FALSE, bar_limits = color_range, fileout = paste0('~/dcpp_map_', chosen_season, '.png'))

} else {
s2dv::PlotEquiMap(diff_clim_obs[1, , ], lon = lons, lat = lats, 
                  toptitle = paste0('SST bias (to ', obs, ') ', exp, ', ', 
                                     vari, ', fyr', chosen_yr, ', ', chosen_season),
                  filled.continents = FALSE, bar_limits = color_range, fileout = paste0('~/dcpp_map_', chosen_season, '.png'))
}
