#Clear workspace
rm(list = ls())

library(maps)
library(RColorBrewer)
library(s2dv)
library(abind)
library(ncdf4)
library(startR)
library(psych)

# Insert variables:
exp <- 'a3w5'
recon <- 'a3t4'  # for Ocean
vari <- 'tosmean'
mod <- 'Omon'
grid <- 'gn'
#QUESTION: all the members? If so, automatically search and get from path; if not, member is a variable
membs_exp <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')
membs_recon <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1')
#QUESTION: all the years? If so, automatically search and get from path; if not, yr1/yr2 is a variable
yr1 <- '1960'
yr2 <- '2020'
reg <- 'Nino3.4' #'Global_Ocean'
int <- 1
#QUESTION: all the time steps? If so, open the first file to get the time step; if not, t1/y2 is a variable
t1 <- 1
t2 <- 132

#===== STEP1: Load CMIP6 DCPP =====

# Variable preparation
# sdate, path
##-----1. If always want the whole period:
#EC-Earth3 is changable (*) only one
#dcppA-hindcast is changable 
first_file_path_till_ver <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/', membs_exp[1], '/',
                                   mod, '/', vari, '/', grid)
#QUESTION: two versions here
ver <- list.files(first_file_path_till_ver)
files <- list.files(paste0(first_file_path_till_ver, '/', ver[2], '/'))
sdates_memb <- unique(sapply(lapply(lapply(files, strsplit, '_'), '[[', 1), '[', 5))
sdates <- sapply(lapply(lapply(sdates_memb, strsplit, '-'), '[[', 1), '[', 1)  #s19601101
path <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/', mod, '/$var$/',
               grid, '/v*/$var$_', mod, '_EC-Earth3_dcppA-hindcast_$sdate$-$memb$_', grid, '_$chunk$.nc')

##-----2. If the period is selected:
#path <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/', mod, '/$var$/',
#               grid, '/v*/$var$_', mod, '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid, '_$chunk$.nc')
#sdates <- paste(seq(yr1, yr2, by = int))

# time
##-----1. If always want the maximal time steps:
# Need to open the first file and get the inner dimensions


# Load data
data <- Start(dat = path,
              var = vari,
              memb = membs_exp,
              region = c(reg),
              sdate = sdates,
              time = c(t1:t2),
              chunk = 'all',
              chunk_depends = 'sdate',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              return_vars = list(time = NULL, region = NULL),
              retrieve = T)

data <- drop(data)
names(dim(data)) <- c("memb", "sdate", "time")
dim(data)


#===== STEP2: Load reconstruction =====
# Variable preparation
# path
path <- paste0('/esarchive/exp/ecearth/', recon, '/diags/CMIP/EC-Earth-Consortium/EC-Earth3-LR/piControl/$memb$/' , mod, '/$var$/',
               grid, '/v*/$var$_', mod, '_EC-Earth3-LR_piControl_$memb$_', grid, '_$chunk$.nc')

data_recon <- Start(dat = path,
              var = vari,
              memb = membs_recon,
              region = c(reg),
              time = 'all',
              chunk = 'all',
              chunk_depends = 'sdate',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              return_vars = list(time = NULL, region = NULL),
              retrieve = T)

#a3t4
data_recon <- InsertDim(drop(data_recon), 2, 1, 'sdate')
names(dim(data_recon)) <- c("memb", "sdate", "time")
dim(data_recon)

#Hist to hind
#QUESTION: why until 2021? exp is till 2020, and sdate = 62 is empty 
recon_sdates <- gsub("-", "", paste(seq(as.Date("1960/11/01"), as.Date("2021/11/01"), "years"), sep = "-"))
nleadt <- dim(data)['time'] #132
data_recon_hc <- Reorder(Histo2Hindcast(data_recon, '19600101', recon_sdates, nleadt, sdate_dim = "sdate", ftime_dim = "time"),
                    c('memb','sdate','time'))
dim(data_recon_hc)


#===== STEP3: Load EC-Earth historical runs =====
#QUESTION: how to store the pre-processing data? And the file name regulation to get the file automatically? Should the data be generated automatically?
nc_file <- paste0('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hist_', vari, '_', reg, '_s1960-s2022.nc')
nc_data <- nc_open(nc_file)
#nc_data <- nc_open('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hist_tosmean_Global_Ocean_s1960-s2022.nc')
data_hist <- ncvar_get(nc_data, "tosmean")
nc_close(nc_data)
names(dim(data_hist)) <- c("memb", "sdate", "time")
dim(data_hist)


#===== STEP4: Load HadISST =====
#QUESTION: same as above
obs <- "hadisst"
nc_file <- paste0('/esarchive/scratch/Earth/rfernand/monitor_dcpp/', obs, '_', vari, '_', reg, '_s1960-s2020.nc')
nc_data <- nc_open(nc_file)
#nc_data <- nc_open('/esarchive/scratch/Earth/rfernand/monitor_dcpp/hadisst_tosmean_Global_Ocean_s1960-s2020.nc')
#QUESTION: The unit transfer and dim adjustment should be done when generating the nc file
data_obs <- InsertDim(ncvar_get(nc_data, "tos"), 1, 1, 'memb') - 273.15
nc_close(nc_data)
names(dim(data_obs)) <- c("memb", "sdate", "time")
dim(data_obs)


#===== STEP5: Compute annual means ===== (only global mean)
#Question: Which variables need to compute annual mean? Which use monthly data directly?
if (vari %in% c('Global_Ocean')) {
    
#exp
data_yr <- Reorder(Season(data, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE),
                   c(2, 3, 1))
dim(data_yr)

#recon
data_recon_yr <- Reorder(Season(data_recon_hc, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE),
                         c(2, 3, 1))
dim(data_recon_yr)

#Historical
data_hist_yr <- Reorder(Season(data_hist, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE), c(2, 3, 1))
dim(data_hist_yr)

#HadISST
data_obs_yr <- Reorder(Season(data_obs, time_dim = 'time', monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE), c(2, 3, 1))
dim(data_obs_yr)
    
}


# Plotting: Timeseries based on monthly means (type 1a)

options(repr.plot.width = 8, repr.plot.height = 6, repr.plot.res = 200)

brks_def <- c(seq(1, dim(data)['sdate'], by = 4))
colors <- readRDS('/esarchive/scratch/Earth/rfernand/dcpp/colorbar_21.RDS')
col_fun <- colorRampPalette(colors[1:20])
jBrewColorsano <- c(col_fun(length(brks_def) - 1))

#PLOT
par(mfrow = c(2, 1), cex = 0.65)
pcexlab <- 1.3
pcexmain <- 1.3
pcexax <- 1.3

t2 <- seq(1, dim(data)['sdate'], by = 1) #seq(1, 61, by = 1)
nf <- 6 # first 6 ftime
#yrs <- seq(yr1, yr2, by = 1) # not used
max_range <- range(range(data[, , 1:nf]), range(data_recon))
enlarge_ratio <- 0.00
ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)

# 1st part
# create the frame and draw the first member
plot(c(1:732), data_recon[1, , 1:732], col = "grey", type = "l", lwd = 0.5, ylim = ylim, xlim = c(1, 350), cex.lab = pcexlab, cex.main = pcexmain, cex.axis = pcexax,
     xlab = "Time (mon)", ylab = "T", main = paste0('a) ', reg, ', ', vari, ', ', exp), lty = 1)

# draw the rest of member
for (i in seq(2, dim(data_recon)['memb'])) {
  lines(c(1:732), data_recon[i, , ], col = "grey", type = "l", lwd = 0.5)
}

# draw exp, memb by memb, sdate by sdate
for (i in seq(1, dim(data)['memb'])) {
  for (t in seq(1, length(t2))) {
#    l <- c(1:nf) + (12 * t2[t] - 2)
    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = 'blue', type = "l", lwd = 0.5) 
    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = jBrewColorsano[t / 4.5 + 1], type = "l", lwd = 0.5)
  }
}

# 2nd part
plot(c(1:732), data_recon[1, , 1:732], col = "grey", type = "l", lwd = 0.5, ylim = ylim, xlim = c(351,732), cex.lab = pcexlab, cex.main = pcexmain, cex.axis = pcexax,
     xlab = "Time (mon)", ylab = "T", main = paste0('a) ', reg, ', ', vari, ', ', exp, ' (cont.)'), lty = 1)

# draw the rest of member
for (i in seq(2, dim(data_recon)['memb'])) {
  lines(c(1:732), data_recon[i, , ], col = "grey", type = "l", lwd = 0.5)
}

# draw exp, memb by memb, sdate by sdate
for (i in seq(1, dim(data)['memb'])) {
  for (t in seq(1, length(t2))) {
#    l <- c(1:nf) + (12 * t2[t] - 2)
    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = 'blue', type = "l", lwd = 0.5) 
    lines(c(1:nf) + (12 * t2[t] - 2), data[i, t, 1:nf], col = jBrewColorsano[t / 4.5 + 1], type = "l", lwd = 0.5)
  }
}


#===== STEP6: Compute Clim and Ano =====

#Extarct same number of startdates

#QUESTION: why 60? Can it be 61 (the shortest sdate)?
sds <- 60 # min(sapply(list(dim(data_yr), dim(data_recon_yr), dim(data_hist_yr), dim(data_obs_yr)), '[', 'sdate'))

monf <- ifelse(reg %in% c('Nino3.4'), 30, dim(data_yr)['time'])

if (reg %in% c('Nino3.4')) {
    data_tmp <- data
    data_recon_tmp <- data_recon_hc
    data_hist_tmp <- data_hist
    data_obs_tmp <- data_obs
} else {
    data_tmp <- data_yr
    data_recon_tmp <- data_recon_yr
    data_hist_tmp <- data_hist_yr
    data_obs_tmp <- data_obs_yr
}

data_yrs <- InsertDim(data_tmp[, 1:sds, 1:monf], 1, 1, 'dat')
data_recon_yrs <- InsertDim(data_recon_tmp[, 1:sds, 1:monf], 1, 1, 'dat')
data_hist_yrs <- InsertDim(data_hist_tmp[, 1:sds, 1:monf], 1, 1, 'dat')
data_obs_yrs <- InsertDim(InsertDim(data_obs_tmp[, 1:sds, 1:monf], 1, 1, 'memb'), 1, 1, 'dat')

# When the bug is fixed...
#data_yrs <- ClimProjDiags::Subset(data_yr, 'sdate', 1:sds, drop = FALSE)
#data_recon_yrs <- ClimProjDiags::Subset(data_recon_yr, 'sdate', 1:sds, drop = FALSE)
#data_hist_yrs <- ClimProjDiags::Subset(data_hist_yr, 'sdate', 1:sds, drop = FALSE)
#data_obs_yrs <- ClimProjDiags::Subset(data_obs_yr, 'sdate', 1:sds, drop = FALSE)

dim(data_yrs)
dim(data_recon_yrs)
dim(data_hist_yrs)
dim(data_obs_yrs)



#Calculate Clim
clim_data <- Clim(data_yrs, data_obs_yrs, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                  method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)

clim_recon <- Clim(data_recon_yrs, data_obs_yrs, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                   method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
clim_hist <- Clim(data_hist_yrs, data_obs_yrs, time_dim = "sdate", dat_dim = c("dat", "memb"),  # dat_dim = "memb",
                  method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)

#Calculate anomalies & ensemble mean
data_yrs_anom <- Ano(data_yrs, clim_data$clim_exp)
dim(data_yrs_anom)
data_yrs_anom_ens <- MeanDims(data_yrs_anom, 'memb')
dim(data_yrs_anom_ens)

data_obs_yrs_anom <- Ano(data_obs_yrs, clim_data$clim_obs)
dim(data_obs_yrs_anom)
data_obs_yrs_anom_ens <- MeanDims(data_obs_yrs_anom, 'memb')
dim(data_obs_yrs_anom_ens)

data_hist_yrs_anom <- Ano(data_hist_yrs, clim_hist$clim_exp)
dim(data_hist_yrs_anom)
data_hist_yrs_anom_ens <- MeanDims(data_hist_yrs_anom, 'memb')
dim(data_hist_yrs_anom_ens)

# QUESTION: the second in put should be clim_data or clim_recon?
data_recon_yrs_anom <- Ano(data_recon_yrs, clim_data$clim_exp)
dim(data_recon_yrs_anom)
data_recon_yrs_anom_ens <- MeanDims(data_recon_yrs_anom, 'memb')
dim(data_recon_yrs_anom_ens)


#===== STEP7: Compute Skill (ACC) =====

acc_data  <- s2dv::Corr(data_yrs_anom_ens, data_obs_yrs_anom_ens, time_dim = 'sdate', dat_dim = 'dat', 
                       comp_dim = NULL, limits = NULL, method = 'pearson', pval = TRUE, conf = TRUE, conf.lev = 0.95, ncores = NULL)

acc_hist  <- s2dv::Corr(data_hist_yrs_anom_ens, data_obs_yrs_anom_ens, time_dim = 'sdate', dat_dim = 'dat', 
                        comp_dim = NULL, limits = NULL, method = 'pearson', pval = TRUE, conf = TRUE, conf.lev = 0.95, ncores = NULL)

#ACC sig
acc_data_sig <- as.integer(acc_data$p.val[1, , ] < 0.05)
acc_data_sig[acc_data_sig == 0] <- NaN

acc_hist_sig <- as.integer(acc_hist$p.val[1, , ] < 0.05)
acc_hist_sig[acc_hist_sig == 0] <- NaN

#ACC diff sig
#NOTE: not used
#ohc_acc_mod  = Corr(tos_a3w5_yrs_anom_ens, tos_hist_yrs_anom_ens, time_dim = 'sdate', dat_dim = 'dat', 
#                 comp_dim = NULL, limits = NULL, method = 'pearson', pval = TRUE, conf = TRUE, conf.lev = 0.95, ncores = NULL)

# Effective sample size
eff_sample <- Eno(data_obs_yrs_anom, time_dim = 'sdate')[1, , ]
# Significance
sig_corr <- paired.r(acc_data$corr[1, , ], acc_hist$corr[1, , ], acc_hist$corr[1, , ], round(eff_sample), twotailed = TRUE)$p < 0.05
sig_corr2 = sig_corr * 1
sig_corr2[sig_corr2 == 0] <- NaN


# Plotting: type II and IV
options(repr.plot.width = 6, repr.plot.height = 5, repr.plot.res = 300)

par(mfrow = c(2, 1), cex = 0.65)

pcexlab <- 1.3
pcexmain <- 1.3
pcexax <- 1.3

#------------ global mean SST, FY1 (Anomaly timeseries (type II)) -------------
yrs <- seq(1, dim(data_yrs_anom_ens)['sdate'], by = 1) + as.numeric(substr(sdates[1], 2, 5)) #yr1 #1960
lt <- 1
max_range <- range(range(data_yrs_anom_ens[1, , lt]), range(data_recon_yrs_anom_ens[1, , lt]), range(data_hist_yrs_anom_ens[1, , lt]), range(data_obs_yrs_anom_ens[1, , lt]))
enlarge_ratio <- 0.0
ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)

plot(yrs + (lt - 1), data_yrs_anom_ens[1, , lt], col = "black", type = "l", lwd = 1.5, ylim = ylim, xlim = range(yrs), cex.lab = pcexlab, 
     cex.main = pcexmain, cex.axis = pcexax, xlab = "Start Date", ylab = "T", main = paste0('a) ', reg, ', ', vari, ', FY1'), lty = 1)

lines(yrs + (lt - 1), data_recon_yrs_anom_ens[1, , lt], col = "grey", lwd = 1.5)
lines(yrs + (lt - 1), data_hist_yrs_anom_ens[1, , lt], col = "blue", lwd = 1.5)
lines(yrs + (lt - 1), data_obs_yrs_anom_ens[1, , lt], col = "red", lwd = 1.5)

# Decide legend position
legend_y_pos <- ifelse(data_yrs_anom_ens[1, 1, lt] > mean(ylim), ylim[1] + (mean(ylim) - ylim[1]) / 1.5, ylim[2]) 
legend(as.numeric(substr(sdates[1], 2, 5)), legend_y_pos, legend = c(exp, recon, 'Hist', obs), col = c("black", 'grey', "blue", "red"), lty = 1, bty = "n")

#------- ACC (type IV) ----------
#NOTE: yrs is different; nino3.4 is 1:30
yrs <- c(1:dim(acc_data$corr)['time'])
plot(yrs, acc_data$corr[1, 1, ], col = "black", type = "l", lwd = 1, ylim = c(0, 1), xlim = range(yrs), cex.lab = pcexlab, cex.main = pcexmain, cex.axis = pcexax,
     xlab = "FYr", ylab = "Cor", main = "b) ACC", lty = 1)

lines(yrs, acc_data$corr[1, 1, ] * acc_data_sig, col = "black", type = "p", lwd = 1.0, pch = 1, cex = 1)
#QUESTION: Two significances?
lines(yrs, acc_data$corr[1, 1, ] * sig_corr2, col = "black", type = "p", lwd = 1.0, pch = 16, cex = 1)
lines(yrs, acc_hist$corr[1, 1, ], col = 'blue', type = "l", lwd = 1)
lines(yrs, acc_hist$corr[1, 1, ] * acc_hist_sig, col = "blue", type = "p", lwd = 1.0, pch = 1, cex = 1)

# Decide legend position
if (acc_data$corr[1, 1, 1] > 0.5 & acc_hist$corr[1, 1, 1] > 0.5) {
    legend_y_pos <- 0.2
} else if (acc_data$corr[1, 1, 1] < 0.5 & acc_hist$corr[1, 1, 1] < 0.5) {
    legend_y_pos <- 0.8
} else {
    legend_y_pos <- 0.5
}
legend(1, legend_y_pos, legend = c(paste0(exp, '-', obs), paste0('Hist-', obs)), col = c("black", "blue"), lty = 1, bty = "n")



#Plotting: Lead-time dependent climatology (type III)
options(repr.plot.width = 6, repr.plot.height = 5, repr.plot.res = 300)
par(mfrow = c(2, 1), cex = 0.65)

pcexlab <- 1.3
pcexmain <- 1.3
pcexax <- 1.3

#Clim
#NOTE: yrs is different; ylim change
yrs <- c(1:dim(clim_data$clim_exp)['time'])
max_range <- range(range(clim_data$clim_exp[1, ]), range(clim_recon$clim_exp[1, ]), range(clim_hist$clim_exp[1, ]), range(clim_data$clim_obs[1, ]))
enlarge_ratio <- 0.2
ylim <- c(max_range[1] - diff(max_range) * enlarge_ratio, max_range[2] + diff(max_range) * enlarge_ratio)

plot(yrs, clim_data$clim_exp[1, ], col = "black", type = "l", lwd = 1, ylim = ylim, xlim = range(yrs), cex.lab=pcexlab, cex.main=pcexmain, cex.axis=pcexax,
     xlab = "FYr", ylab = "T", main = paste0("a) Climatology, ", reg, ', ', vari), lty = 1)
#QUESTION: Wrong data/color?
lines(yrs, clim_recon$clim_exp[1, ], col = 'grey', type = "l", lwd = 1)
lines(yrs, clim_hist$clim_exp[1, ], col = "blue", type = "l", lwd = 1)
lines(yrs, clim_data$clim_obs[1, ], col = 'red', type = "l", lwd = 1)

# Decide legend position
legend_y_pos <- ifelse(clim_data$clim_exp[1, 1] > mean(ylim), ylim[1] + (mean(ylim) - ylim[1]) / 1.5, ylim[2])
legend(1, legend_y_pos, legend = c(exp, recon, 'Hist', obs), col = c("black", "grey", "blue", "red"), lty = 1, bty = "n")



