 fun_Season_diffClim_ACC <- function(data, data_obs, chosen_mon) {
    # data: [sdate, time, memb]
    # data_obs: [sdate, time]
    # chosen_mon: c(moninf, monsup)
    data_yrs <- apply(data, c(1, 3), s2dv:::.Season, monini = 1, moninf = chosen_mon[1], monsup = chosen_mon[2], method = mean, na.rm = TRUE)
    data_obs_yrs <- apply(data_obs, 1, s2dv:::.Season, monini = 1, moninf = chosen_mon[1], monsup = chosen_mon[2], method = mean, na.rm = TRUE)
    #data_yrs should be [time, sdate, memb], data_obs_yrs should be [time, sdate]
    if (length(dim(data_yrs)) != 3) {
      data_yrs <- array(data_yrs, dim = c(time = 1, dim(data_yrs)['sdate'], dim(data_yrs)['memb']))
    }
    if (length(dim(data_obs_yrs)) != 2) {
      data_obs_yrs <- array(data_obs_yrs, dim = c(time = 1, sdate = length(data_obs_yrs)))
    }
#    ====CHECK====
#    if (any(names(dim(data_yrs)) != c('time', 'sdate', 'memb')) |
#        any(names(dim(data_obs_yrs)) != c('time', 'sdate'))) {
#      stop("fun_Season_diffClim_ACC error 1.")
#    }

    # Calculate Clim
    clim_data <- array(dim = c(dim(data_yrs)[1]))
    clim_data_obs <- array(dim = c(dim(data_yrs)[1]))
    for (tt in 1:dim(data_yrs)[1]) { #loop through time dim
      clim_data[tt] <- mean(data_yrs[tt, , ])
      clim_data_obs[tt] <- mean(data_obs_yrs[tt, ])
    }

#    # Add dim for Clim (before dat_dim can be NULL)
#    data_yrs <- s2dv::InsertDim(data_yrs, 3, 1, 'dat')
#    data_obs_yrs <- s2dv::InsertDim(s2dv::InsertDim(data_obs_yrs, 3, 1, 'dat'), 4, 1, 'memb')
#    # Now data_yrs and data_obs_yrs should be [time, sdate, dat, memb]
#
#    clim_data <- array(dim = c(time = dim(data_yrs)[1]))
#    clim_data_obs <- array(dim = c(time = dim(data_yrs)[1]))
#    for (tt in 1:dim(data_yrs)[1]) {
#      data_tmp <- ClimProjDiags::Subset(data_yrs, 1, tt, drop = 'selected')
#      data_obs_tmp <- ClimProjDiags::Subset(data_obs_yrs, 1, tt, drop = 'selected')
#      tmp <- s2dv:::.Clim(data_tmp, data_obs_tmp,
#                          time_dim = 'sdate', method = "clim", dat_dim = c('dat', 'memb'),
#                          memb_dim = 'memb', memb = FALSE, na.rm = TRUE)
#      clim_data[tt] <- tmp$clim_exp
#      clim_data_obs[tt] <- tmp$clim_obs
#    }

    # result 1: diff (clim - obs)
    diff <- clim_data - clim_data_obs

    #======================================
    # Calculate Ano
    data_yrs_list <- plyr::alply(data_yrs, 2:3)
    data_yrs_anom <- array(unlist(lapply(data_yrs_list, '-', clim_data)),
                           dim = dim(data_yrs))
    data_obs_yrs_list <- plyr::alply(data_obs_yrs, 2)
    data_obs_yrs_anom <- array(unlist(lapply(data_obs_yrs_list, '-', clim_data_obs)),
                               dim = dim(data_obs_yrs))
print('CHECK anom:')
print(dim(data_yrs_anom))
print(dim(data_obs_yrs_anom))

    # Calculate ensemble mean
    data_yrs_anom_ens <- apply(data_yrs_anom, 1:2, mean)
    if (length(dim(data_yrs_anom_ens)) != 2) {
      data_yrs_anom_ens <- array(data_yrs_anom_ens, dim = c(time = 1, dim(data_yrs_anom)['sdate']))
    }
    #data_yrs_anom_ens and data_obs_yrs_anom should be [time, sdate]

print('CHECK anom_ens:')
print(dim(data_yrs_anom_ens))

    # Calculate ACC
    corr <- array(dim = dim(data_yrs_anom_ens)[1]) #time dim
    eno <- array(dim = dim(data_yrs_anom_ens)[1])
    for (i_time in 1:dim(data_yrs_anom_ens)[1]) {
      corr[i_time] <- cor(data_yrs_anom_ens[i_time, ], data_obs_yrs_anom[i_time, ],
                          use = "pairwise.complete.obs",
                          method = 'pearson')
      eno[i_time] <- s2dv:::.Eno(data_obs_yrs_anom[i_time, ], na.action = na.pass)

    }
    # corr and eno should be [time]

    # ACC sig
    t <- sqrt(corr * corr * (eno - 2) / (1 - (corr ^ 2)))
    p.val <- pt(t, eno - 2, lower.tail = FALSE)
    # p.val should be [time]
    acc_data_sig <- as.integer(p.val < 0.05)
    acc_data_sig[acc_data_sig == 0] <- NaN
    dim(acc_data_sig) <- c(time = length(acc_data_sig))

    return(list(diff = diff, acc = corr, acc_sig = acc_data_sig))
  }
