library(startR)

# exp
repos_exp <- paste0('/esarchive/exp/ecearth/a1tr/cmorfiles/CMIP/EC-Earth-Consortium/',
                    'EC-Earth3/historical/r24i1p1f1/Amon/$var$/gr/v20190312/',
                    '$var$_Amon_EC-Earth3_historical_r24i1p1f1_gr_$sdate$01-$sdate$12.nc')

exp <- Start(dat = repos_exp,
             var = 'tas',
             sdate = as.character(c(2005:2008)),
             time = indices(1:3),
             lat = 'all',
             lon = 'all',
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'sdate'),
             retrieve = FALSE)

lats <- attr(exp, 'Variables')$common$lat
lons <- attr(exp, 'Variables')$common$lon
## The 'time' attribute is a two-dim array
dates <- attr(exp, 'Variables')$common$time
dim(dates)
#sdate  time 
#    4     3 

repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$date$.nc'

obs <- Start(dat = repos_obs,
             var = 'tas',
#             sdate = unique(format(dates, '%Y%m')),
             date = unique(format(dates, '%Y%m')),
             time = values(dates),  #dim: [sdate = 4, time = 3]
             lat = values(lats), # indices(256:1),
             lon = values(lons), # 'all',
#             time_across = 'sdate',
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL),
             retrieve = FALSE)


#------------------------------------------------------------------

repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
time <- array(dim = c(sdate = 3, time = 2)) # sdate can be whatever. time needs to be time
time[1, 1] <- as.POSIXct('2005-01-16 12:00:00', tz = 'UTC')
time[1, 2] <- as.POSIXct('2005-01-31 12:00:00', tz = 'UTC')
time[2, 1] <- as.POSIXct('2006-01-16 12:00:00', tz = 'UTC')
time[2, 2] <- as.POSIXct('2006-01-31 12:00:00', tz = 'UTC')
time[3, 1] <- as.POSIXct('2007-01-16 12:00:00', tz = 'UTC')
time[3, 2] <- as.POSIXct('2007-01-31 12:00:00', tz = 'UTC')
time2<- as.POSIXct(time, origin = '1970-01-01', tz = 'UTC')

obs <- Start(dat = repos_obs,
             var = 'tas',
             sdate = c('200501', '200601', '200701'),
#             date = unique(format(dates, '%Y%m')),
             time = time2,  #dim: [sdate = 3, time = 2]
             lat = values(lats), # indices(256:1),
             lon = values(lons), # 'all',
             time_across = 'sdate',
#             time_across = 'date',
#             merge_across_dims = TRUE,
#             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL),
             retrieve = T)

dim(obs)
# dat  var  sdate time  lat  lon 
#   1    1      3    2  256  512 

#------------------------------------------------------------------

repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$date$.nc'

obs <- Start(dat = repos_obs,
             var = 'tas',
             date = c('200501', '200601', '200701'),
#             date = unique(format(dates, '%Y%m')),
             time = time2,  #dim: [sdate = 3, time = 2]
             lat = values(lats), # indices(256:1),
             lon = values(lons), # 'all',
             time_across = 'date',
#             time_across = 'date',
#             merge_across_dims = TRUE,
#             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL),
             retrieve = T)

dim(obs)
# dat  var  date time  lat  lon 
#   1    1     3    2  256  512 

