#library(startR)
require(startR, lib.loc = "/shared/earth/software/R/3.6.1-foss-2015a-bare/lib64/R/library") #change the loaded library


expid <- 'a3vg'
model <- 'EC-Earth3-HR'
compo  <- 'Omon'
grid <- 'gn'
vari <- 'tosmean'
reg <- 'Nino3'
startdates <- as.character(c(seq(1990,1992)))
init_month <- 5
length_forecast <- 4
membs_num <- c(seq(1,2))

forecast_month <- c(seq(init_month,(init_month+length_forecast-1)))
membs <- paste0('r', membs_num, 'i1p1f1', sep="")

path <- paste0('/esarchive/exp/ecearth/', expid, 
               '/diags/DCPP/EC-Earth-Consortium/', model, 
               '/dcppA-hindcast/$memb$/', compo, '/$var$/', grid, 
               '/v*/$var$_', compo, '_', model, 
               '_dcppA-hindcast_s$sdate$-$memb$_', grid, '_$chunk$.nc')

data <- Start(dat = path,
                          var = vari,
                          memb = membs,
                          region = reg,
                          region_var = 'region',
                          sdate = startdates,
                          time = c(1:length(forecast_month)), 
                          chunk = 'all',
                          chunk_depends = 'sdate',
                          time_across = 'chunk', 
                          merge_across_dims = TRUE,
                          return_vars = list(time = c('sdate','chunk'), 
                                             region=NULL, tosmean = NULL), 
                          retrieve = F)

str(attr(attr(data,"Variables")$common$tosmean, 'variables')$tosmean$units)
# chr "degC"


fun <- function(x) {
x
}

step <- Step(fun, target_dims = 'memb', output_dims = 'memb')

wf <- AddStep(data, step)

#res <- Compute(wf, chunks = list(time = 1))

#-------------------user-defined---------------------
  queue_host <- 'nord1'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------

  res <- Compute(wf,
                 chunks = list(time = 2),
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = queue_host,
                   queue_type = 'lsf',
                   temp_dir = temp_dir,
                   cores_per_job = 2,
                   job_wallclock = '05:00',
                   max_jobs = 4,
                   extra_queue_params = list('#BSUB -q bsc_es'),
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = TRUE
                 )  #$output1

