library(startR)
library(ClimProjDiags)

# exp
repos_exp <- paste0('/esarchive/exp/ecearth/a1tr/cmorfiles/CMIP/EC-Earth-Consortium/',
                    'EC-Earth3/historical/r24i1p1f1/Amon/$var$/gr/v20190312/',
                    '$var$_Amon_EC-Earth3_historical_r24i1p1f1_gr_$sdate$01-$sdate$12.nc')

exp <- Start(dat = repos_exp,
             var = 'tas',
             sdate = as.character(c(2005:2008)),
             time = indices(1:3),
             lat = 'all',
             lon = 'all',
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'sdate'),
             retrieve = FALSE)
lats <- attr(exp, 'Variables')$common$lat
lons <- attr(exp, 'Variables')$common$lon
## The 'time' attribute is a two-dim array
dates <- attr(exp, 'Variables')$common$time
dim(dates)
#sdate  time 
#    4     3


# obs
repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$date$.nc'

obs <- Start(dat = repos_obs,
             var = 'tas',
             date = unique(format(dates, '%Y%m')),
             time = values(dates),  #dim: [sdate = 4, time = 3]
             lat = values(lats), # indices(256:1),
             lon = values(lons), # 'all',
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'date'),
             retrieve = FALSE)

attr(obs, 'Dimensions')
#  dat   var sdate  time   lat   lon 
#    1     1     4     3   256   512 

fun <- function(x) {
#print(dim(x))
#print(aa)
return(x)
}
step <- Step(fun, target_dims = 'time', output_dims = 'time')
wf <- AddStep(obs, step)
res <- Compute(wf, chunks = list(sdate = 2))

dim(res$output1)
# time   dat   var sdate   lat   lon 
#    3     1     1     4   256   512 

res2 <- Compute(wf, chunks = list(sdate = 1))
dim(res2$output1)
# time   dat   var sdate   lat   lon 
#    3     1     1     4   256   512 
identical(res$output1, res2$output1)
#[1] TRUE

#########################################################33


ecmwf_path_hc <- paste0('/esarchive/exp/ecmwf/s2s-monthly_ensforhc/daily/$var$_f24h/$sdate$/$var$_$syear$.nc')
obs_path <-paste0('/esarchive/recon/ecmwf/era5/daily/$var$-r240x121/$var$_$file_date$.nc')

sdates.seq <- c("20161222","20161229", "20170105","20170112")

hcst <- Start(dat = ecmwf_path_hc,
              var = 'sfcWind',
              sdate = sdates.seq,
              syear =  indices(1:2), #'all',
              time = 'all',
              latitude = indices(1),
              longitude = indices(1),
              ensemble = indices(1:2), #'all',
              syear_depends = 'sdate',
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = c('sdate','syear')
                                  ),
              retrieve = F)
datess <- attr(hcst, 'Variables')$common$time
file_date <- unique(sapply(datess, format, '%Y%m'))

obs <- Start(dat = obs_path,
             var = 'windagl100',
             latitude = indices(1),
             longitude = indices(1),
             file_date= file_date,
             time = values(datess),    # 'sdate' 'syear' 'time'
             time_var = 'time',
             time_across = 'file_date',
             merge_across_dims= TRUE,
             merge_across_dims_narm = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(latitude=c('lat','latitude'),
                             longitude=c('lon','longitude')),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',#
                                time = c('file_date')),
             retrieve = F)

fun <- function(x) {
print(dim(x))
#print(aa)
return(x)
}
st <- Step(fun, target_dims = 'time', output_dims = 'time')
adst <- AddStep(obs, st)
res <- Compute(adst, chunks = list(sdate = 2))

