
vari_obs <- 'tos'
vari <- 'tosmean'
membs_num <- c(seq(1,2))
reg <- 'Nino3.4'
lat_min = -5
lat_max = 5
lon_min = -170
lon_max = -120 
init_month <- 5
startdates <- as.character(c(seq(1990,1995)))
length_forecast <- 4

membs <- paste0('r', membs_num, 'i1p1f1', sep="")
forecast_month <- c(seq(init_month,(init_month+length_forecast-1)))
nstartdates <- length(startdates)

#####################
# my function
fun_spatial_mean2 <- function(x, lons_data, lats_data, lon_min, lon_max, lat_min, lat_max){
    data_smean <- ClimProjDiags::WeightedMean(data = x, 
                               lon = lons_data, 
                               lat=lats_data,
                               region = c(lon_min, lon_max, lat_min, lat_max), 
                               mask = NULL, 
                               londim = which(names(dim(x)) == 'lon'),
                               latdim = which(names(dim(x)) == 'lat'))
    return(data_smean)
}
target_dims = c('lat','lon')
output_dims = c(NULL) 
step2 <- Step(fun = fun_spatial_mean2,
              target_dims = target_dims, 
              output_dims = output_dims)
            
#####################
# Load experiment
path <- paste0('/esarchive/exp/ecearth/a3vg/diags/DCPP/EC-Earth-Consortium/EC-Earth3-HR/dcppA-hindcast/$memb$/Omon/$var$/gn/v*/$var$_Omon_EC-Earth3-HR_dcppA-hindcast_s$sdate$-$memb$_gn_$chunk$.nc')
print(paste0('path of the exp: ', path))
data_exp_retrieved <- Start(dat = path,
                                    var = vari,
                                    memb = membs,
                                    region = reg,
                                    region_var = 'region',
                                    sdate = startdates,
                                    time = c(1:length(forecast_month)), 
                                    chunk = 'all',
                                    chunk_depends = 'sdate',
                                    time_across = 'chunk',
                                    merge_across_dims = TRUE,
                                    return_vars = list(time=c('sdate','chunk'), region=NULL), 
                                    retrieve = F)

####################
# 1. Load observations with dates from Experiment's attributes
####################
dates_to_use <- drop(attr(data_exp_retrieved, 'Variables')$common$time)
print('Load observations')

repos_obs <- '/esarchive/obs/ukmo/hadisst_v1.1/monthly_mean/tos/$var$_$date$.nc'
print(repos_obs)
data_obs_retrieved <- Start(dat = repos_obs,
                                    var = vari_obs,
                                    date = unique(format(dates_to_use, '%Y%m')), 
                                    time = values(dates_to_use), 
                                    lat = 'all',
                                    lat_reorder = Sort(decreasing = FALSE), 
                                    lon = 'all', 
                                    lon_reorder = CircularSort(-180, 180),
                                    synonims = list(lon = c('lon', 'longitude'), 
                                                    lat = c('lat', 'latitude')),
                                    time_across = 'date', 
                                    merge_across_dims = TRUE,
                                    merge_across_dims_narm = TRUE,
                                    split_multiselected_dims = TRUE,
                                    return_vars = list(lat='dat', lon='dat', time=c('date')),
                                    retrieve = F)
print('Dimension of non-retrieved obs: ')
print(attr(data_obs_retrieved, 'Dimension'))
lat_obs = as.vector(attr(data_obs_retrieved, 'Variables')$dat1$lat)
lon_obs = as.vector(attr(data_obs_retrieved, 'Variables')$dat1$lon)
  

wf_obs <- AddStep(inputs = data_obs_retrieved, 
                              step_fun = step2,
                              lons_data = lon_obs,
                              lats_data = lat_obs,
                              lon_min = lon_min,
                              lon_max = lon_max,
                              lat_min = lat_min,
                              lat_max = lat_max)
chunks <- list(sdate = as.numeric(attr(data_obs_retrieved,'Dimensions')['sdate']))

             
machine_call <- 'local'
             
  data_obs_retrieve <- Compute(workflow = wf_obs, chunks = chunks)$output1


 # Nord3-v2
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
  res <- Compute(wf_obs,
                 chunks = chunks, 
                 threads_load = 1,
                 threads_compute = 2,
                 cluster = list(queue_host = 'nord4',
                   queue_type = 'slurm',
                   temp_dir = temp_dir,
                   cores_per_job = 4,
                   job_wallclock = '01:00:00',
                   max_jobs = 4,
 #                   extra_queue_params = list('#BSUB -q bsc_es'),
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = TRUE
                 )  #$output1

