library(startR)

lonmin <- -270
lonmax <-  50
latmin <- -60
latmax <-  50

sdates.test <-  paste0(c(2001:2002), '0501')

# exp
repos_a33d <-  paste0('/esarchive/scratch/eexarcho/Eleftheria/TRIATLAS/Analysis_a33d_a33e_a33f/',
                     'Data/a33d/$var$/',
                     '$var$_Omon_EC-Earth3-CC_historical_s$sdate$-$member$_r360x180_$chunk$.nc')  

a33d <- Start(dat = repos_a33d,
             var = 'chlos',
             member = 'all',
             sdate = sdates.test,
             chunk = 'all',
             time = indices(1:12),  #first time step per day
             chunk_depends = 'sdate',
             time_across = 'chunk',
             merge_across_dims = TRUE,
             lat = values(list(latmin, latmax)),
             lat_reorder = Sort(), ##1
             lon = values(list(lonmin, lonmax)),
             lon_reorder = CircularSort(-180, 180), ##2
            # transform = CDORemapper,
            # transform_extra_cells = 2,
            # transform_params = list(grid = 'r360x180',
            #                         method = 'conservative',
            #                         crop = c(lonmin, lonmax, latmin, latmax)),
            # transform_vars = c('lat', 'lon'),
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = 'dat',  ##3
                                lat = 'dat',  ##3
                                time = 'sdate'),
             retrieve = T)


lons <- as.vector(attr(a33d, 'Variables')$dat1$lon)
lats <- as.vector(attr(a33d, 'Variables')$dat1$lat)
dates <- attr(a33d, 'Variables')$common$time
dates_file <- sort(unique(gsub('-', '', sapply(as.character(dates), substr, 1, 7))))


#-------------------------------------
# obs

repos_obs <- '/esarchive/obs/esa/occci-v5.0/monthly_mean/$var$/$var$_$date$.nc'

obs.data <- Start(dat = repos_obs,
             var = 'chlos',
             date = dates_file,
             time = values(dates),  #dim: [sdate = 2, time = 12]
             time_across = 'date',
             merge_across_dims = TRUE,
             merge_across_dims_narm = TRUE,
             split_multiselected_dims = TRUE,
             lon_reorder = CircularSort(-180, 180),
             lat_reorder = Sort(),
             lat = values(list(latmin, latmax)),
             lon = values(list(lonmin, lonmax)),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = "/esarchive/scratch/eexarcho/Eleftheria/TRIATLAS/Analysis_a33d_a33e_a33f/Data/a33d/chlos/chlos_Omon_EC-Earth3-CC_historical_s20010501-r10i1p1f1_r360x180_200105-200204.nc",
                                     method = 'bil',
                                     crop = c(lonmin, lonmax, latmin, latmax)),
             transform_vars = c('lat', 'lon'),
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lat = NULL,  ##3
                                lon = NULL,  ##3
                                time = 'date'),
             retrieve = TRUE)

#============================================

library(easyNCDF)
file <- NcOpen('/esarchive/obs/esa/occci-v5.0/monthly_mean/chlos/chlos_200105.nc')

obs <- NcToArray(file, vars_to_read = 'chlos',
                  dim_indices = list(lon = c(1:1440),
                                     lat = c(1:720),
                                     time = c(1)))
latss <- NcToArray(file,
                  dim_indices = list(lat = 1:720), vars_to_read = 'lat')
lonss <- NcToArray(file,
                  dim_indices = list(lon = 1:1440), vars_to_read = 'lon')
#lonss[201]
#[1] 50
#lonss[361]
#[1] 90

NcClose(file)


#----------------------------------------------------

obs2 <- Start(dat = repos_obs,
             var = 'chlos',
             date = dates_file,
             time = values(dates),  #dim: [sdate = 2, time = 12]
             time_across = 'date',
             merge_across_dims = TRUE,
             merge_across_dims_narm = TRUE,
             split_multiselected_dims = TRUE,
             lon_reorder = CircularSort(-180, 180),
             lat_reorder = Sort(),
             lat = values(list(latmin, latmax)),
             lon = values(list(lonmin, lonmax)),
#             transform = CDORemapper,
#             transform_extra_cells = 2,
#             transform_params = list(grid = "/esarchive/scratch/eexarcho/Eleftheria/TRIATLAS/Analysis_a33d_a33e_a33f/Data/a33d/chlos/chlos_Omon_EC-Earth3-CC_historical_s20010501-r10i1p1f1_r360x180_200105-200204.nc",
#                                     method = 'bil',
#                                     crop = c(lonmin, lonmax, latmin, latmax)),
#             transform_vars = c('lat', 'lon'),
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lat = NULL,  ##3
                                lon = NULL,  ##3
                                time = 'date'),
             retrieve = TRUE)

lons2 <- attr(obs2, 'Variables')$common$lon
lats2 <- attr(obs2, 'Variables')$common$lat

dim(obs2)
#  dat   var sdate  time   lat   lon 
#    1     1     2    12   440  1281 

s2dv::PlotEquiMap(obs2[1,1,1,1,,], lon = lons2, lat = lats2, filled.continents = F)

# crop the easyNCDF output
lonss2 <- lonss[c(1:201, 361:1440)] # 0 to 50, 90 to 360
latss2 <- latss[121:560] # -59.875 to 49.875
s2dv::PlotEquiMap(obs[1,c(1:201, 361:1440), 121:560,1], lon = lonss2, lat = latss2,
                  filled.continents = F)


