#-----------------------------------
# Script to bias correct predictions 
# ECMWF ERA5 & SEAS5
# 
# Chung
# July 2021
#-----------------------------------
rm(list = ls())
vars <- c('prlr', 'tas', 'tasmax', 'tasmin')
seasons <- c('DJF', 'NDJ', 'OND', 'SON', 'ASO', 'JAS',
             'JJA', 'MJJ', 'AMJ', 'MAM', 'FMA', 'JFM')
var <- 1
season <- 6
print(paste0('Bias correct ', seasons[season], ' ', vars[var], ' ...'))

library(startR)

path.output <- '/esarchive/scratch/cchou/FOCUS/teleconnection/output/'

# exp
path.exp <- list('/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_s0-24h/$var$_$sdate$.nc',
		 '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc', 
		 '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f24h/$var$_$sdate$.nc',
		 '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f24h/$var$_$sdate$.nc')

# obs
path.obs <- list('/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$date$.nc',
		 '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$date$.nc',
		 '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f24h-r1440x721cds/$var$_$date$.nc',
		 '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f24h-r1440x721cds/$var$_$date$.nc')

y1 <- 1981; y2 <- 2016; months <- 12:1
leadtimemin <- 1; leadtimemax <- 3
lons.min <- 0; lons.max <- 60; lats.min <- -40; lats.max <- 10 # SA domain

ini_month <- sprintf("%02d", months[season])
sdate <- paste0(y1:y2, ini_month, '01')

exp <- Start(data = path.exp[[var]],
             var = vars[var],
             member = 'all',
             sdate = sdate,
             time = leadtimemin:leadtimemax,
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = TRUE),
             longitude = values(list(lons.min, lons.max)),
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude'),
                             member = c('member', 'ensemble')),
             return_vars = list(longitude = NULL,
                                latitude = NULL,
                                time = 'sdate'),
             transform = CDORemapper,
#             transform_extra_cells = 2,
             transform_params = list(grid = 'r1440x721',
                                     method = 'bicubic',
                                     crop = c(lons.min, lons.max,
					      lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             retrieve = FALSE)

# manually create the dates for obs

if (ini_month == '12') {
        dates.obs <- c(paste0(y1:y2, ini_month, '15'),
                       paste0((y1+1):(y2+1), '01', '15'),
                       paste0((y1+1):(y2+1), '02', '15')) }

if (ini_month == '11') {
        dates.obs <- c(paste0(y1:y2, ini_month, '15'),
                       paste0(y1:y2, months[season] + 1, '15'),
                       paste0((y1+1):(y2+1), '01', '15')) }

if (ini_month %in% c('10','09','08','07','06','05','04','03','02','01')) {
        dates.obs <- c(paste0(y1:y2, ini_month, '15'),
                       paste0(y1:y2, sprintf("%02d", months[season] + 1), '15'),
                       paste0(y1:y2, sprintf("%02d", months[season] + 2), '15')) }

# convert to the form required by Start()

time.obs <- as.POSIXct(dates.obs, "%Y%m%d",
                       origin = paste0(y1,'-', ini_month), tz = 'UTC')
dim(time.obs) <- c('sdate' = y2 - y1 + 1, 'time' = 3)


obs <- Start(data = path.obs[[var]],
             var = vars[var],
             date = unique(format(time.obs, '%Y%m')),
             time = values(time.obs),
             latitude = indices(1:2), #values(list(lats.min, lats.max)),
	     latitude_reorder = Sort(decreasing = TRUE),
             longitude = indices(1:2), #values(list(lons.min, lons.max)),
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
             return_vars = list(longitude = NULL,
                                latitude = NULL,
                                time = 'date'),
             retrieve = FALSE)


#----------------------
fun <- function (obs) {
  return(obs)
}
step <- Step(fun, target_dims = list(obs = 'sdate'), output_dims = list(obs = 'sdate'))
wf <- AddStep(obs, step)
res <- Compute(wf, chunks = list(time = 3))
#----------------------


