library(startR)

exp <- 'a3w5'
vari <- 'tas'
mod <- 'Amon'
grid <- '1x1'
t1 <- 1
t2 <- 24
yr1 <- '1981'
yr2 <- '1982'
lats <- c(-10, 10)
lons <- c(0, 20)
membs <- 'r1i1p1f1'

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------
repos <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/',
                mod, '/$var$/', grid, '/v*/$var$_', mod,
                '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid,
                '_$chunk$.nc')

data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = paste(seq(yr1, yr2)),
              chunk = 'all',
              chunk_depends = 'sdate',
              time = c(t1:t2),
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
#              latitude = 'all',
              latitude_reorder = Sort(),
#              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
#              transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = 'r360x180', method = 'con', crop = FALSE), transform_vars = c('lat','lon'),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              retrieve = FALSE)
attr(data, 'Dimensions')



#--------------------------
# Step3: Load obs
#--------------------------
obs <- 'era5'
path_obs <- paste0('/esarchive/recon/ecmwf/', obs, '/monthly_mean/$var$_f1h/$var$_$file_date$.nc')
time_obs <- attr(data, 'Variables')$common$time #[sdate, time]
data_obs <- Start(dat = path_obs,
              var = vari,
              file_date = unique(format(time_obs, '%Y%m')), #paste(seq(yr1, yr2)),
              time = values(time_obs),
              time_across = 'file_date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
              transform = CDORemapper, transform_extra_cells = 2,
              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',
                                      method = 'con'),
              transform_vars = c('latitude', 'longitude'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'),
                                 latitude = NULL, longitude = NULL),
              retrieve = FALSE)
attr(data_obs, 'Dimensions')


 fun <- function(data, data_obs) {
   return(mean(data, na.rm = T))
 }

 step <- Step(fun = fun,
              target_dims = list(data = c('sdate', 'time', 'memb'),
                                 data_obs = c('sdate', 'time')),
               output_dims = NULL)

  wf <- AddStep(inputs = list(data, data_obs),
                step_fun = step)

 # local
  output <- Compute(workflow = wf,
                    chunks = list(longitude = 2))



 fun2 <- function(data_obs) {
   return(mean(data_obs, na.rm = T))
 }
 step2 <- Step(fun = fun2,
              target_dims = c('sdate', 'time'), output_dims = NULL)
  wf2 <- AddStep(inputs = data_obs,
                step_fun = step2)
output <- Compute(workflow = wf2, chunks = list(longitude = 2))
output2 <- Compute(workflow = wf2, chunks = list(longitude = 1))

