
lats <- c(-90, 90)
lons <- c(0, 359.9)
membs <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------

repos <- paste0('/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/Amon/$var$/1x1/v*/$var$_Amon_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_1x1_$chunk$.nc')

m1 <- gc()

data <- Start(dat = repos,
              var = 'tas',
              memb = membs,
              sdate = paste(seq(1979, 2018)),
              chunk = 'all',
              chunk_depends = 'sdate',
              time = c(1:6), #'all',
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
#              latitude = 'all',
              latitude_reorder = Sort(),
#              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              retrieve = F)
m2 <- gc()
m2 - m1

m2-m1
#         used (Mb) gc trigger (Mb) max used (Mb)
#Ncells  65309  3.5          0    0   643252 34.3
#Vcells 273792  2.1          0    0  2547291 19.4

# installed startR
m2-m1
#         used (Mb) gc trigger (Mb) max used (Mb)
#Ncells  72060  3.8          0    0   376751 20.1
#Vcells 179654  1.4          0    0  1205907  9.2

attr(data, 'Dimensions')


#--------------------------
# Step3: Load obs
#--------------------------
path_obs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r360x180/$var$_$file_date$.nc')
time_data <- attr(data, 'Variables')$common$time #[sdate, time]
m3 <- gc()
data_obs <- Start(dat = path_obs,
              var = 'tas',
              file_date = unique(format(time_data, '%Y%m')), #paste(seq(yr1, yr2)),
              time = values(time_data),
              time_across = 'file_date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'),
                                 latitude = NULL, longitude = NULL),
              retrieve = F)
m4 <- gc()

m4-m3
#        used (Mb) gc trigger (Mb)  max used   (Mb)
#Ncells  5809  0.3          0    0         0    0.0
#Vcells 26278  0.2  136837240 1044 176264120 1344.8

# installed startR
m4-m3
#        used (Mb) gc trigger (Mb) max used (Mb)
#Ncells  2856  0.2          0    0        0    0
#Vcells 12417  0.1          0    0        0    0


attr(data_obs, 'Dimensions')
