
library(SpecsVerification)

grid_file <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_20140701.nc'

lons.min <- 20
lons.max <- 40
lats.min <- 20
lats.max <- 40

exp <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(2000:2010,'0101'),
             ensemble = 'all',
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             longitude = values(list(lons.min, lons.max)),
             transform = CDORemapper,
             transform_params = list(grid =grid_file,
                                     method = 'con', crop = c(lons.min,lons.max,lats.min,lats.max)),
             transform_vars = c('latitude', 'longitude'),
             synonims = list(latitude=c('lat','latitude'),longitude=c('longitude','lon')),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time=c('sdate')),
             retrieve= F)

obs <- Start(dat = '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(2000:2010,'02'),
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = T),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_params = list(grid =grid_file,
                                     method = 'con', crop = c(lons.min,lons.max,lats.min,lats.max)),
             transform_vars = c('latitude', 'longitude'),
#             transform_extra_cells = 8,
             synonims = list(latitude=c('lat','latitude'),longitude=c('longitude','lon')),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time=c('sdate')),
             merge_across_dims = TRUE,
             retrieve= F)


# Define function (applied on the essential 'target' dimensions)
compute_scores <- function(exp, obs) {
  my.prob <- c(1/3, 2/3)
  my.FairRpss <- veriApply("FairRpss", fcst=exp, obs=obs,
                           prob=my.prob, tdim=1, ensdim=2, na.rm=TRUE)[[1]]
  dim(my.FairRpss) <- c(score = length(my.FairRpss))
  return(my.FairRpss)
}
step <- Step(compute_scores,
             target_dims = list(exp=c('sdate','ensemble'),
                                obs=c('sdate')),
             output_dims = list(my.FairRpss = 'score'),
             use_libraries = c('easyVerification',
                               'SpecsVerification'))

# workflow of operations
wf <- AddStep(list(exp, obs), step)

res <- Compute(wf,
               chunks = list(latitude = 2, longitude = 2, time = 1))

#######

fun <- function(exp, obs) {
#print(dim(exp))
#print(dim(obs))
print(aaa)
return(list(exp,obs))
}
step <- Step(fun, target_dims = list(exp = c('sdate', 'ensemble'), 
                                     obs = c('sdate')),
             output_dims= list(exp = c('time', 'sdate', 'ensemble'), 
                               obs = c('time', 'sdate')))
wf <- AddStep(list(exp, obs), step)

res1 <- Compute(wf$exp, chunk = list(longitude = 2))
res2 <- Compute(wf$exp, chunk = list(latitude = 2))

#######

fun <- function(x) {
return(x)
}

step <- Step(fun, target_dims = c('sdate', 'ensemble'),
             output_dims = c('sdate', 'ensemble'))
wf <- AddStep(exp, step)

step <- Step(fun, target_dims = c('sdate'),
             output_dims = c('sdate'))
wf <- AddStep(obs, step)

res3 <- Compute(wf, chunk = list(longitude = 2))


#===================================================================
lons.min <- 10
lons.max <- 20
lats.min <- 20
lats.max <- 40

exp <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = '20000101', #paste0(2000:2001, '0101'),
             ensemble = indices(1), #'all',
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_params = list(grid = 'r100x50', #grid_file,
                                     method = 'con', 
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             transform_extra_cells = 8,
             synonims = list(latitude = c('lat', 'latitude'), 
                             longitude = c('longitude', 'lon')),
             return_vars = list(latitude = NULL, #'dat',
                                longitude = NULL, #'dat',
                                time = 'sdate'),
             retrieve= F)

func <- function(exp) {
  return(exp)
}
step <- Step(func,
             target_dims = 'sdate', output_dims = 'sdate')
wf <- AddStep(exp, step)
res <- Compute(wf,
               chunks = list(latitude = 2))$output1

    sdate       dat       var  ensemble      time  latitude longitude 
        1         1         1         1         1        10         3 

#sdate = 20000101
drop(res)
          [,1]     [,2]     [,3]
 [1,] 284.9907 285.4820 286.1208
 [2,] 282.9883 282.9362 284.3523
 [3,] 281.1704 281.9748 284.1095
 [4,]       NA       NA       NA
 [5,]       NA       NA       NA
 [6,]       NA       NA       NA
 [7,]       NA       NA       NA
 [8,] 281.2961 282.7828 286.4284
 [9,] 284.1387 287.3716 287.7389
[10,] 285.6547 285.0194 286.1099

## transform_extra_cells = 5
drop(res)
          [,1]     [,2]     [,3]
 [1,] 284.9907 285.4820 286.1208
 [2,] 282.9883 282.9362 284.3523
 [3,] 281.1279 282.2220 285.0612
 [4,]       NA       NA       NA
 [5,]       NA       NA       NA
 [6,]       NA       NA       NA
 [7,] 281.2128 282.0873 284.0594
 [8,] 281.2574 282.6088 285.9198
 [9,] 284.1387 287.3716 287.7389
[10,] 285.6547 285.0194 286.1099


#---------------------
res1 <- Compute(wf,
               chunks = list(ensemble = 1))$output1

dim(res1$output1)
    sdate       dat       var  ensemble      time  latitude longitude 
        2         1         1         1         1         5         6 

#sdate = 20000101
drop(res1$output1)[,]
         [,1]     [,2]     [,3]
[1,] 284.9907 285.4820 286.1208
[2,] 282.9883 282.9362 284.3523
[3,] 281.2574 282.6088 285.9198
[4,] 284.1387 287.3716 287.7389
[5,] 285.6547 285.0194 286.1099

