library(startR)
library(s2dverification)

path <- '/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/Amon/$var$/gr/v20190713/$var$_Amon_*_s$sdate$-$member$_gr_$leadyear$.nc'

data <- Start(dataset = path,
              var = 'tas',
              member = indices(1),
              sdate = '2000', #paste0(2000:2004),
              leadyear = indices(1:3), #'all',
#              leadyear_depends = 'sdate',
              month = 'all', #indices(1:25), #'all',
              month_across = 'leadyear',
#              merge_across_dims = TRUE,  # without this, Compute returns wrong dim
              lat = indices(1), #values(list(0, 14)),
              lat_reorder = Sort(),
              lon = indices(1), #values(list(0, 28)),
              lon_reorder = CircularSort(0, 360),
              synonims = list(month = c('month', 'time'), 
                              lon = c('lon', 'longitude'), 
                              lat = c('lat', 'latitude')),
#              transform = CDORemapper,
#              transform_extra_cells = 2,
#              transform_params = list(grid = 'r200x100',
#                                      method = 'con', 
#                                      crop = c(0, 28, 0, 14)),
#              transform_vars = c('lat', 'lon'),
              return_vars = list(lat = 'dataset', lon = 'dataset'),
              retrieve = FALSE)

attr(data, 'Dimensions')
# dataset      var   member    sdate leadyear    month      lat      lon 
#       1        1        2        5       11       12        8       16 

fun <- function(exp){
  return(exp)
}

step <- Step(fun = fun,
#             use_libraries = c('s2dverification'),
             target_dims = c('lat','lon'),
             output_dims = c('lat','lon'))

wf <- AddStep(inputs = data, step = step)

res <- Compute(workflow = wf,
               chunks = list(member = 2, month = 2),
               threads_load = 2, threads_compute = 4)

# correct
res2 <- Compute(workflow = wf,
                chunks = list(member = 2))
dim(res2$output1)
#     lat      lon  dataset      var   member    sdate leadyear    month 
#       8       16        1        1        2        5       11       12 

res3 <- Compute(workflow = wf, chunks = list(month = 2))
dim(res3$output1)
#     lat      lon  dataset      var   member    sdate leadyear    month 
#       8       16        1        1        2        5        6       24 

res4 <- Compute(workflow = wf,
                chunks = list(leadyear = 2))

res5 <- Compute(workflow = wf,
                chunks = list(leadyear = 2, month = 2))
